# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Abiquo Test Suite
"""
import sys

from libcloud.test import MockHttp, unittest
from libcloud.utils.py3 import ET, httplib
from libcloud.common.types import LibcloudError, InvalidCredsError
from libcloud.compute.base import NodeImage, NodeLocation
from libcloud.test.compute import TestCaseMixin
from libcloud.common.abiquo import ForbiddenError, get_href
from libcloud.test.file_fixtures import ComputeFileFixtures
from libcloud.compute.drivers.abiquo import AbiquoNodeDriver


class AbiquoNodeDriverTest(TestCaseMixin, unittest.TestCase):
    """
    Abiquo Node Driver test suite
    """

    @classmethod
    def setUpClass(cls):
        """
        Set up the driver with the main user
        """
        AbiquoNodeDriver.connectionCls.conn_class = AbiquoMockHttp
        cls.driver = AbiquoNodeDriver("son", "goku", "http://dummy.host.com/api")

    def test_unauthorized_controlled(self):
        """
        Test the Unauthorized Exception is Controlled.

        Test, through the 'login' method, that a '401 Unauthorized'
        raises a 'InvalidCredsError' instead of the 'MalformedUrlException'
        """
        self.assertRaises(
            InvalidCredsError,
            AbiquoNodeDriver,
            "son",
            "goten",
            "http://dummy.host.com/api",
        )

    def test_forbidden_controlled(self):
        """
        Test the Forbidden Exception is Controlled.

        Test, through the 'list_images' method, that a '403 Forbidden'
        raises an 'ForbidenError' instead of the 'MalformedUrlException'
        """
        AbiquoNodeDriver.connectionCls.conn_class = AbiquoMockHttp
        conn = AbiquoNodeDriver("son", "gohan", "http://dummy.host.com/api")
        self.assertRaises(ForbiddenError, conn.list_images)

    def test_handle_other_errors_such_as_not_found(self):
        """
        Test common 'logical' exceptions are controlled.

        Test that common exception (normally 404-Not Found and 409-Conflict),
        that return an XMLResponse with the explanation of the errors are
        controlled.
        """
        self.driver = AbiquoNodeDriver("go", "trunks", "http://dummy.host.com/api")
        self.assertRaises(LibcloudError, self.driver.list_images)

    def test_ex_create_and_delete_empty_group(self):
        """
        Test the creation and deletion of an empty group.
        """
        group = self.driver.ex_create_group("libcloud_test_group")
        group.destroy()

    def test_create_node_no_image_raise_exception(self):
        """
        Test 'create_node' without image.

        Test the 'create_node' function without 'image' parameter raises
        an Exception
        """
        self.assertRaises(TypeError, self.driver.create_node)

    def test_list_locations_response(self):
        if not self.should_list_locations:
            return None

        locations = self.driver.list_locations()
        self.assertTrue(isinstance(locations, list))

    def test_create_node_specify_location(self):
        """
        Test you can create a node specifying the location.
        """
        image = self.driver.list_images()[0]
        location = self.driver.list_locations()[0]
        self.driver.create_node(image=image, location=location)

    def test_create_node_specify_wrong_location(self):
        """
        Test you can not create a node with wrong location.
        """
        image = self.driver.list_images()[0]
        location = NodeLocation(435, "fake-location", "Spain", self.driver)
        self.assertRaises(LibcloudError, self.driver.create_node, image=image, location=location)

    def test_create_node_specify_wrong_image(self):
        """
        Test image compatibility.

        Some locations only can handle a group of images, not all of them.
        Test you can not create a node with incompatible image-location.
        """
        # Create fake NodeImage
        image = NodeImage(3234, "dummy-image", self.driver)
        location = self.driver.list_locations()[0]
        # With this image, it should raise an Exception
        self.assertRaises(LibcloudError, self.driver.create_node, image=image, location=location)

    def test_create_node_specify_group_name(self):
        """
        Test 'create_node' into a concrete group.
        """
        image = self.driver.list_images()[0]
        self.driver.create_node(image=image, ex_group_name="new_group_name")

    def test_create_group_location_does_not_exist(self):
        """
        Test 'create_node' with an unexistent location.

        Defines a 'fake' location and tries to create a node into it.
        """
        location = NodeLocation(435, "fake-location", "Spain", self.driver)
        # With this location, it should raise an Exception
        self.assertRaises(
            LibcloudError,
            self.driver.ex_create_group,
            name="new_group_name",
            location=location,
        )

    def test_destroy_node_response(self):
        """
        'destroy_node' basic test.

        Override the destroy to return a different node available
        to be undeployed. (by default it returns an already undeployed node,
        for test creation).
        """
        self.driver = AbiquoNodeDriver("go", "trunks", "http://dummy.host.com/api")
        node = self.driver.list_nodes()[0]
        ret = self.driver.destroy_node(node)
        self.assertTrue(ret)

    def test_destroy_node_response_failed(self):
        """
        'destroy_node' asynchronous error.

        Test that the driver handles correctly when, for some reason,
        the 'destroy' job fails.
        """
        self.driver = AbiquoNodeDriver("muten", "roshi", "http://dummy.host.com/api")
        node = self.driver.list_nodes()[0]
        ret = self.driver.destroy_node(node)
        self.assertFalse(ret)

    def test_destroy_node_allocation_state(self):
        """
        Test the 'destroy_node' invalid state.

        Try to destroy a node when the node is not running.
        """
        self.driver = AbiquoNodeDriver("ve", "geta", "http://dummy.host.com/api")
        # Override the destroy to return a different node available to be
        # undeployed
        node = self.driver.list_nodes()[0]
        # The mock class with the user:password 've:geta' returns a node that
        # is in 'ALLOCATION' state and hence, the 'destroy_node' method should
        # raise a LibcloudError
        self.assertRaises(LibcloudError, self.driver.destroy_node, node)

    def test_destroy_not_deployed_group(self):
        """
        Test 'ex_destroy_group' when group is not deployed.
        """
        location = self.driver.list_locations()[0]
        group = self.driver.ex_list_groups(location)[1]
        self.assertTrue(group.destroy())

    def test_destroy_deployed_group(self):
        """
        Test 'ex_destroy_group' when there are machines running.
        """
        location = self.driver.list_locations()[0]
        group = self.driver.ex_list_groups(location)[0]
        self.assertTrue(group.destroy())

    def test_destroy_deployed_group_failed(self):
        """
        Test 'ex_destroy_group' fails.

        Test driver handles correctly when, for some reason, the
        asynchronous job fails.
        """
        self.driver = AbiquoNodeDriver("muten", "roshi", "http://dummy.host.com/api")
        location = self.driver.list_locations()[0]
        group = self.driver.ex_list_groups(location)[0]
        self.assertFalse(group.destroy())

    def test_destroy_group_invalid_state(self):
        """
        Test 'ex_destroy_group' invalid state.

        Test the Driver raises an exception when the group is in
        invalid temporal state.
        """
        self.driver = AbiquoNodeDriver("ve", "geta", "http://dummy.host.com/api")
        location = self.driver.list_locations()[0]
        group = self.driver.ex_list_groups(location)[1]
        self.assertRaises(LibcloudError, group.destroy)

    def test_run_node(self):
        """
        Test 'ex_run_node' feature.
        """
        node = self.driver.list_nodes()[0]
        # Node is by default in NodeState.TERMINATED and AbiquoState ==
        # 'NOT_ALLOCATED'
        # so it is available to be ran
        self.driver.ex_run_node(node)

    def test_run_node_invalid_state(self):
        """
        Test 'ex_run_node' invalid state.

        Test the Driver raises an exception when try to run a
        node that is in invalid state to run.
        """
        self.driver = AbiquoNodeDriver("go", "trunks", "http://dummy.host.com/api")
        node = self.driver.list_nodes()[0]
        # Node is by default in AbiquoState = 'ON' for user 'go:trunks'
        # so is not available to be ran
        self.assertRaises(LibcloudError, self.driver.ex_run_node, node)

    def test_run_node_failed(self):
        """
        Test 'ex_run_node' fails.

        Test driver handles correctly when, for some reason, the
        asynchronous job fails.
        """
        self.driver = AbiquoNodeDriver("ten", "shin", "http://dummy.host.com/api")
        node = self.driver.list_nodes()[0]
        # Node is in the correct state, but it fails because of the
        # async task and it raises the error.
        self.assertRaises(LibcloudError, self.driver.ex_run_node, node)

    def test_get_href(self):
        xml = """
<datacenter>
        <link href="http://10.60.12.7:80/api/admin/datacenters/2"
        type="application/vnd.abiquo.datacenter+xml" rel="edit1"/>
        <link href="http://10.60.12.7:80/ponies/bar/foo/api/admin/datacenters/3"
        type="application/vnd.abiquo.datacenter+xml" rel="edit2"/>
        <link href="http://vdcbridge.interoute.com:80/jclouds/apiouds/api/admin/enterprises/1234"
        type="application/vnd.abiquo.datacenter+xml" rel="edit3"/>
</datacenter>
"""

        elem = ET.XML(xml)

        href = get_href(element=elem, rel="edit1")
        self.assertEqual(href, "/admin/datacenters/2")
        href = get_href(element=elem, rel="edit2")
        self.assertEqual(href, "/admin/datacenters/3")
        href = get_href(element=elem, rel="edit3")
        self.assertEqual(href, "/admin/enterprises/1234")


class AbiquoMockHttp(MockHttp):

    """
    Mock the functionality of the remote Abiquo API.
    """

    fixtures = ComputeFileFixtures("abiquo")
    fixture_tag = "default"

    def _api_login(self, method, url, body, headers):
        if headers["Authorization"] == "Basic c29uOmdvdGVu":
            expected_response = self.fixtures.load("unauthorized_user.html")
            expected_status = httplib.UNAUTHORIZED
        else:
            expected_response = self.fixtures.load("login.xml")
            expected_status = httplib.OK
        return (expected_status, expected_response, {}, "")

    def _api_cloud_virtualdatacenters(self, method, url, body, headers):
        return (httplib.OK, self.fixtures.load("vdcs.xml"), {}, "")

    def _api_cloud_virtualdatacenters_4(self, method, url, body, headers):
        return (httplib.OK, self.fixtures.load("vdc_4.xml"), {}, "")

    def _api_cloud_virtualdatacenters_4_virtualappliances(self, method, url, body, headers):
        if method == "POST":
            vapp_name = ET.XML(body).findtext("name")
            if vapp_name == "libcloud_test_group":
                # we come from 'test_ex_create_and_delete_empty_group(self):'
                # method and so, we return the 'ok' return
                response = self.fixtures.load("vdc_4_vapp_creation_ok.xml")
                return (httplib.OK, response, {}, "")
            elif vapp_name == "new_group_name":
                # we come from 'test_ex_create_and_delete_empty_group(self):'
                # method and so, we return the 'ok' return
                response = self.fixtures.load("vdc_4_vapp_creation_ok.xml")
                return (httplib.OK, response, {}, "")
        else:
            # It will be a 'GET';
            return (httplib.OK, self.fixtures.load("vdc_4_vapps.xml"), {}, "")

    def _api_cloud_virtualdatacenters_4_virtualappliances_5(self, method, url, body, headers):
        if method == "GET":
            if headers["Authorization"] == "Basic dmU6Z2V0YQ==":
                # Try to destroy a group with 'needs_sync' state
                response = self.fixtures.load("vdc_4_vapp_5_needs_sync.xml")
            else:
                # Try to destroy a group with 'undeployed' state
                response = self.fixtures.load("vdc_4_vapp_5.xml")
            return (httplib.OK, response, {}, "")
        else:
            # it will be a 'DELETE'
            return (httplib.NO_CONTENT, "", {}, "")

    def _api_cloud_virtualdatacenters_4_virtualappliances_6(self, method, url, body, headers):
        if method == "GET":
            # deployed vapp
            response = self.fixtures.load("vdc_4_vapp_6.xml")
            return (httplib.OK, response, {}, "")
        else:
            # it will be a 'DELETE'
            return (httplib.NO_CONTENT, "", {}, "")

    def _api_cloud_virtualdatacenters_4_virtualappliances_6_virtualmachines_3_tasks_1da8c8b6_86f6_49ef_9d29_57dcc73b875a(
        self, method, url, body, headers
    ):
        if headers["Authorization"] == "Basic bXV0ZW46cm9zaGk=":
            # User 'muten:roshi' failed task
            response = self.fixtures.load("vdc_4_vapp_6_undeploy_task_failed.xml")
        else:
            response = self.fixtures.load("vdc_4_vapp_6_undeploy_task.xml")
        return (httplib.OK, response, {}, "")

    def _api_cloud_virtualdatacenters_4_virtualappliances_5_virtualmachines(
        self, method, url, body, headers
    ):
        # This virtual app never have virtual machines
        if method == "GET":
            response = self.fixtures.load("vdc_4_vapp_5_vms.xml")
            return (httplib.OK, response, {}, "")
        elif method == "POST":
            # it must be a POST
            response = self.fixtures.load("vdc_4_vapp_6_vm_creation_ok.xml")
            return (httplib.CREATED, response, {}, "")

    def _api_cloud_virtualdatacenters_4_virtualappliances_6_virtualmachines(
        self, method, url, body, headers
    ):
        # Default-created virtual app virtual machines'
        if method == "GET":
            if headers["Authorization"] == "Basic dmU6Z2V0YQ==":
                response = self.fixtures.load("vdc_4_vapp_6_vms_allocated.xml")
            else:
                response = self.fixtures.load("vdc_4_vapp_6_vms.xml")
            return (httplib.OK, response, {}, "")
        else:
            # it must be a POST
            response = self.fixtures.load("vdc_4_vapp_6_vm_creation_ok.xml")
            return (httplib.CREATED, response, {}, "")

    def _api_cloud_virtualdatacenters_4_virtualappliances_6_virtualmachines_3(
        self, method, url, body, headers
    ):
        if (
            headers["Authorization"] == "Basic Z286dHJ1bmtz"
            or headers["Authorization"] == "Basic bXV0ZW46cm9zaGk="
        ):
            # Undeploy node
            response = self.fixtures.load("vdc_4_vapp_6_vm_3_deployed.xml")
        elif headers["Authorization"] == "Basic dmU6Z2V0YQ==":
            # Try to undeploy a node with 'allocation' state
            response = self.fixtures.load("vdc_4_vapp_6_vm_3_allocated.xml")
        else:
            # Get node
            response = self.fixtures.load("vdc_4_vapp_6_vm_3.xml")
        return (httplib.OK, response, {}, "")

    def _api_cloud_virtualdatacenters_4_virtualappliances_6_virtualmachines_3_action_deploy(
        self, method, url, body, headers
    ):
        response = self.fixtures.load("vdc_4_vapp_6_vm_3_deploy.xml")
        return (httplib.CREATED, response, {}, "")

    def _api_cloud_virtualdatacenters_4_virtualappliances_6_virtualmachines_3_tasks_b44fe278_6b0f_4dfb_be81_7c03006a93cb(
        self, method, url, body, headers
    ):
        if headers["Authorization"] == "Basic dGVuOnNoaW4=":
            # User 'ten:shin' failed task
            response = self.fixtures.load("vdc_4_vapp_6_vm_3_deploy_task_failed.xml")
        else:
            response = self.fixtures.load("vdc_4_vapp_6_vm_3_deploy_task.xml")
        return (httplib.OK, response, {}, "")

    def _api_cloud_virtualdatacenters_4_virtualappliances_6_action_undeploy(
        self, method, url, body, headers
    ):
        response = self.fixtures.load("vdc_4_vapp_6_undeploy.xml")
        return (httplib.OK, response, {}, "")

    def _api_cloud_virtualdatacenters_4_virtualappliances_6_virtualmachines_3_action_reset(
        self, method, url, body, headers
    ):
        response = self.fixtures.load("vdc_4_vapp_6_vm_3_reset.xml")
        return (httplib.CREATED, response, {}, "")

    def _api_cloud_virtualdatacenters_4_virtualappliances_6_virtualmachines_3_tasks_a8c9818e_f389_45b7_be2c_3db3a9689940(
        self, method, url, body, headers
    ):
        if headers["Authorization"] == "Basic bXV0ZW46cm9zaGk=":
            # User 'muten:roshi' failed task
            response = self.fixtures.load("vdc_4_vapp_6_undeploy_task_failed.xml")
        else:
            response = self.fixtures.load("vdc_4_vapp_6_vm_3_reset_task.xml")
        return (httplib.OK, response, {}, "")

    def _api_cloud_virtualdatacenters_4_virtualappliances_6_virtualmachines_3_action_undeploy(
        self, method, url, body, headers
    ):
        response = self.fixtures.load("vdc_4_vapp_6_vm_3_undeploy.xml")
        return (httplib.CREATED, response, {}, "")

    def _api_cloud_virtualdatacenters_4_virtualappliances_6_virtualmachines_3_network_nics(
        self, method, url, body, headers
    ):
        response = self.fixtures.load("vdc_4_vapp_6_vm_3_nics.xml")
        return (httplib.OK, response, {}, "")

    def _api_admin_datacenters(self, method, url, body, headers):
        return (httplib.OK, self.fixtures.load("dcs.xml"), {}, "")

    def _api_admin_enterprises_1(self, method, url, body, headers):
        return (httplib.OK, self.fixtures.load("ent_1.xml"), {}, "")

    def _api_admin_enterprises_1_datacenterrepositories(self, method, url, body, headers):
        # When the user is the common one for all the tests ('son, 'goku')
        # it creates this basic auth and we return the datacenters  value
        if headers["Authorization"] == "Basic Z286dHJ1bmtz":
            expected_response = self.fixtures.load("not_found_error.xml")
            return (httplib.NOT_FOUND, expected_response, {}, "")
        elif headers["Authorization"] != "Basic c29uOmdvaGFu":
            return (httplib.OK, self.fixtures.load("ent_1_dcreps.xml"), {}, "")
        else:
            # son:gohan user: forbidden error
            expected_response = self.fixtures.load("privilege_errors.html")
            return (httplib.FORBIDDEN, expected_response, {}, "")

    def _api_admin_enterprises_1_datacenterrepositories_2(self, method, url, body, headers):
        return (httplib.OK, self.fixtures.load("ent_1_dcrep_2.xml"), {}, "")

    def _api_admin_enterprises_1_datacenterrepositories_2_virtualmachinetemplates(
        self, method, url, body, headers
    ):
        return (httplib.OK, self.fixtures.load("ent_1_dcrep_2_templates.xml"), {}, "")

    def _api_admin_enterprises_1_datacenterrepositories_2_virtualmachinetemplates_11(
        self, method, url, body, headers
    ):
        return (httplib.OK, self.fixtures.load("ent_1_dcrep_2_template_11.xml"), {}, "")


if __name__ == "__main__":
    sys.exit(unittest.main())



# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Libcloud Python 2.x and 3.x compatibility layer
# Some methods below are taken from Django PYK3 port which is licensed under 3
# clause BSD license
# https://bitbucket.org/loewis/django-3k

# pylint: skip-file


import sys
import types

DEFAULT_LXML = False

try:
    if DEFAULT_LXML:
        from lxml import etree as ET
    else:
        from xml.etree import ElementTree as ET
except ImportError:
    from xml.etree import ElementTree as ET

PY2 = sys.version_info[0] == 2
PY3 = sys.version_info[0] == 3
PY2_pre_279 = PY2 and sys.version_info < (2, 7, 9)

PY2 = False
PY27 = False
PY3 = False

if (2, 0) <= sys.version_info < (3, 0):
    PY2 = True

if (2, 7) <= sys.version_info < (2, 8):
    PY27 = True

PY3 = True

if PY3:
    import base64
    import urllib
    import urllib as urllib2
    import http.client as httplib

    # pylint: disable=no-name-in-module
    import urllib.parse as urlparse
    import xmlrpc.client as xmlrpclib
    from io import StringIO
    from os.path import relpath
    from urllib.parse import quote as urlquote
    from urllib.parse import unquote as urlunquote
    from urllib.parse import urlencode as urlencode

    if sys.version_info >= (3, 5, 0):
        from importlib import reload
    else:
        from imp import reload

    from builtins import next, bytes

    parse_qs = urlparse.parse_qs
    parse_qsl = urlparse.parse_qsl

    basestring = str

    def method_type(callable, instance, klass):
        return types.MethodType(callable, instance or klass())

    def b(s):
        if isinstance(s, str):
            return s.encode("utf-8")
        elif isinstance(s, bytes):
            return s
        elif isinstance(s, int):
            return bytes([s])
        else:
            raise TypeError("Invalid argument {!r} for b()".format(s))

    def ensure_string(s):
        if isinstance(s, str):
            return s
        elif isinstance(s, bytes):
            return s.decode("utf-8")
        else:
            raise TypeError("Invalid argument {!r} for ensure_string()".format(s))

    def byte(n):
        # assume n is a Latin-1 string of length 1
        return ord(n)

    _real_unicode = str
    u = str

    def bchr(s):
        """Take an integer and make a 1-character byte string."""
        return bytes([s])

    def dictvalues(d):
        return list(d.values())

    def tostring(node):
        return ET.tostring(node, encoding="unicode")

    def hexadigits(s):
        # s needs to be a byte string.
        return [format(x, "02x") for x in s]

    if sys.version_info >= (3, 1, 0):
        # encodestring and decodestring has been deprecated since 3.1.0
        def base64_encode_string(*args, **kwargs):
            return base64.encodebytes(*args, **kwargs)  # NOQA

        def base64_decode_string(*args, **kwargs):
            return base64.decodebytes(*args, **kwargs)  # NOQA

    else:

        def base64_encode_string(*args, **kwargs):
            return base64.encodestring(*args, **kwargs)

        def base64_decode_string(*args, **kwargs):
            return base64.decodestring(*args, **kwargs)

    def assertRaisesRegex(self, *args, **kwargs):
        import unittest

        if not isinstance(self, unittest.TestCase):
            raise ValueError(
                'First argument "self" needs to be an instance ' "of unittest.TestCase"
            )
        return getattr(self, "assertRaisesRegex")(*args, **kwargs)

    def assertRegex(self, *args, **kwargs):
        import unittest

        if not isinstance(self, unittest.TestCase):
            raise ValueError(
                'First argument "self" needs to be an instance ' "of unittest.TestCase"
            )

        return getattr(self, "assertRegex")(*args, **kwargs)

else:
    import base64  # NOQA
    import urllib  # NOQA
    from urllib import quote as _urlquote  # NOQA
    from urllib import unquote as urlunquote  # NOQA
    from urllib import urlencode as urlencode  # NOQA

    import httplib  # NOQA
    import urllib2  # NOQA
    import urlparse  # NOQA
    import xmlrpclib  # NOQA
    from StringIO import StringIO  # NOQA
    from __builtin__ import reload  # NOQA

    parse_qs = urlparse.parse_qs
    parse_qsl = urlparse.parse_qsl

    from os.path import relpath  # NOQA

    # Save the real value of unicode because urlquote needs it to tell the
    # difference between a unicode string and a byte string.
    _real_unicode = unicode  # NOQA
    basestring = unicode = str

    method_type = types.MethodType

    b = bytes = ensure_string = str

    def byte(n):
        return n

    u = unicode

    def bchr(s):
        """Take an integer and make a 1-character byte string."""
        return chr(s)

    _default_value_next = object()

    def next(iterator, default=_default_value_next):
        try:
            return iterator.next()
        except StopIteration:
            if default is _default_value_next:
                raise
            return default

    def dictvalues(d):
        return d.values()

    tostring = ET.tostring

    def urlquote(s, safe="/"):
        if isinstance(s, _real_unicode):
            # Pretend to be py3 by encoding the URI automatically.
            s = s.encode("utf8")
        return _urlquote(s, safe)

    def hexadigits(s):
        # s needs to be a string.
        return [x.encode("hex") for x in s]

    def base64_encode_string(*args, **kwargs):
        return base64.encodestring(*args, **kwargs)

    def base64_decode_string(*args, **kwargs):
        return base64.decodestring(*args, **kwargs)

    def assertRaisesRegex(self, *args, **kwargs):
        import unittest

        if not isinstance(self, unittest.TestCase):
            raise ValueError(
                'First argument "self" needs to be an instance ' "of unittest.TestCase"
            )

        return getattr(self, "assertRaisesRegex")(*args, **kwargs)

    def assertRegex(self, *args, **kwargs):
        import unittest

        if not isinstance(self, unittest.TestCase):
            raise ValueError(
                'First argument "self" needs to be an instance ' "of unittest.TestCase"
            )

        return getattr(self, "assertRegex")(*args, **kwargs)



# Licensed to the Apache Software Foundation (ASF) under one or more§
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import os
import sys
import tempfile
from unittest.mock import Mock, MagicMock, call, patch

from libcloud import _init_once
from libcloud.test import LibcloudTestCase, unittest
from libcloud.utils.py3 import StringIO, u, assertRaisesRegex
from libcloud.compute.ssh import ParamikoSSHClient, ShellOutSSHClient, have_paramiko

if not have_paramiko:
    ParamikoSSHClient = None  # NOQA
    paramiko_version = ()
else:
    import paramiko

    paramiko_version = tuple(int(x) for x in paramiko.__version__.split("."))


@unittest.skipIf(not have_paramiko, "Skipping because paramiko is not available")
class ParamikoSSHClientTests(LibcloudTestCase):
    @patch("paramiko.SSHClient", Mock)
    def setUp(self):
        """
        Creates the object patching the actual connection.
        """
        conn_params = {
            "hostname": "dummy.host.org",
            "port": 8822,
            "username": "ubuntu",
            "key": "~/.ssh/ubuntu_ssh",
            "timeout": "600",
        }
        _, self.tmp_file = tempfile.mkstemp()
        os.environ["LIBCLOUD_DEBUG"] = self.tmp_file
        _init_once()
        self.ssh_cli = ParamikoSSHClient(**conn_params)

    def tearDown(self):
        if "LIBCLOUD_DEBUG" in os.environ:
            del os.environ["LIBCLOUD_DEBUG"]

    @patch("paramiko.SSHClient", Mock)
    def test_create_with_password(self):
        conn_params = {
            "hostname": "dummy.host.org",
            "username": "ubuntu",
            "password": "ubuntu",
        }
        mock = ParamikoSSHClient(**conn_params)
        mock.connect()

        expected_conn = {
            "username": "ubuntu",
            "password": "ubuntu",
            "allow_agent": False,
            "hostname": "dummy.host.org",
            "look_for_keys": False,
            "port": 22,
        }
        mock.client.connect.assert_called_once_with(**expected_conn)
        self.assertLogMsg("Connecting to server")

    @patch("paramiko.SSHClient", Mock)
    def test_deprecated_key_argument(self):
        conn_params = {
            "hostname": "dummy.host.org",
            "username": "ubuntu",
            "key": "id_rsa",
        }
        mock = ParamikoSSHClient(**conn_params)
        mock.connect()

        expected_conn = {
            "username": "ubuntu",
            "allow_agent": False,
            "hostname": "dummy.host.org",
            "look_for_keys": False,
            "key_filename": "id_rsa",
            "port": 22,
        }
        mock.client.connect.assert_called_once_with(**expected_conn)
        self.assertLogMsg("Connecting to server")

    def test_key_files_and_key_material_arguments_are_mutual_exclusive(self):
        conn_params = {
            "hostname": "dummy.host.org",
            "username": "ubuntu",
            "key_files": "id_rsa",
            "key_material": "key",
        }

        expected_msg = "key_files and key_material arguments are mutually " "exclusive"
        assertRaisesRegex(self, ValueError, expected_msg, ParamikoSSHClient, **conn_params)

    @patch("paramiko.SSHClient", Mock)
    def test_key_material_argument(self):
        path = os.path.join(os.path.dirname(__file__), "fixtures", "misc", "test_rsa.key")

        with open(path) as fp:
            private_key = fp.read()

        conn_params = {
            "hostname": "dummy.host.org",
            "username": "ubuntu",
            "key_material": private_key,
        }
        mock = ParamikoSSHClient(**conn_params)
        mock.connect()

        pkey = paramiko.RSAKey.from_private_key(StringIO(private_key))
        expected_conn = {
            "username": "ubuntu",
            "allow_agent": False,
            "hostname": "dummy.host.org",
            "look_for_keys": False,
            "pkey": pkey,
            "port": 22,
        }
        mock.client.connect.assert_called_once_with(**expected_conn)
        self.assertLogMsg("Connecting to server")

    @patch("paramiko.SSHClient", Mock)
    def test_key_material_argument_invalid_key(self):
        conn_params = {
            "hostname": "dummy.host.org",
            "username": "ubuntu",
            "key_material": "id_rsa",
        }

        mock = ParamikoSSHClient(**conn_params)

        expected_msg = "Invalid or unsupported key type"
        assertRaisesRegex(self, paramiko.ssh_exception.SSHException, expected_msg, mock.connect)

    @patch("paramiko.SSHClient", Mock)
    @unittest.skipIf(
        paramiko_version >= (2, 7, 0),
        "New versions of paramiko support OPENSSH key format",
    )
    def test_key_file_non_pem_format_error(self):
        path = os.path.join(
            os.path.dirname(__file__), "fixtures", "misc", "test_rsa_non_pem_format.key"
        )

        # Supplied as key_material
        with open(path) as fp:
            private_key = fp.read()

        conn_params = {
            "hostname": "dummy.host.org",
            "username": "ubuntu",
            "key_material": private_key,
        }

        mock = ParamikoSSHClient(**conn_params)

        expected_msg = "Invalid or unsupported key type"
        assertRaisesRegex(self, paramiko.ssh_exception.SSHException, expected_msg, mock.connect)

    @patch("paramiko.SSHClient", Mock)
    def test_password_protected_key_no_password_provided_1(self):
        path = os.path.join(
            os.path.dirname(__file__),
            "fixtures",
            "misc",
            "test_rsa_2048b_pass_foobar.key",
        )

        # Supplied as key_material
        with open(path) as fp:
            private_key = fp.read()

        conn_params = {
            "hostname": "dummy.host.org",
            "username": "ubuntu",
            "key_material": private_key,
        }

        mock = ParamikoSSHClient(**conn_params)

        expected_msg = "private key file is encrypted"
        assertRaisesRegex(
            self,
            paramiko.ssh_exception.PasswordRequiredException,
            expected_msg,
            mock.connect,
        )

        conn_params = {
            "hostname": "dummy.host.org",
            "username": "ubuntu",
            "key_files": path,
        }

        mock = ParamikoSSHClient(**conn_params)

        expected_msg = "private key file is encrypted"
        assertRaisesRegex(
            self,
            paramiko.ssh_exception.PasswordRequiredException,
            expected_msg,
            mock.connect,
        )

    @patch("paramiko.SSHClient", Mock)
    def test_password_protected_key_no_password_provided_2(self):
        path = os.path.join(
            os.path.dirname(__file__),
            "fixtures",
            "misc",
            "test_rsa_2048b_pass_foobar.key",
        )

        # Supplied as key_material
        with open(path) as fp:
            private_key = fp.read()

        conn_params = {
            "hostname": "dummy.host.org",
            "username": "ubuntu",
            "key_material": private_key,
            "password": "invalid",
        }

        mock = ParamikoSSHClient(**conn_params)

        expected_msg = "OpenSSH private key file checkints do not match"
        assertRaisesRegex(self, paramiko.ssh_exception.SSHException, expected_msg, mock.connect)

    @patch("paramiko.SSHClient", Mock)
    def test_password_protected_key_valid_password_provided(self):
        path = os.path.join(
            os.path.dirname(__file__),
            "fixtures",
            "misc",
            "test_rsa_2048b_pass_foobar.key",
        )

        # Supplied as key_material
        with open(path) as fp:
            private_key = fp.read()

        conn_params = {
            "hostname": "dummy.host.org",
            "username": "ubuntu",
            "key_material": private_key,
            "password": "foobar",
        }

        mock = ParamikoSSHClient(**conn_params)
        self.assertTrue(mock.connect())

        conn_params = {
            "hostname": "dummy.host.org",
            "username": "ubuntu",
            "key_files": path,
            "password": "foobar",
        }

        mock = ParamikoSSHClient(**conn_params)
        self.assertTrue(mock.connect())

    @patch("paramiko.SSHClient", Mock)
    def test_ed25519_key_type(self):
        path = os.path.join(os.path.dirname(__file__), "fixtures", "misc", "test_ed25519.key")

        # Supplied as key_material
        with open(path) as fp:
            private_key = fp.read()

        conn_params = {
            "hostname": "dummy.host.org",
            "username": "ubuntu",
            "key_material": private_key,
        }

        mock = ParamikoSSHClient(**conn_params)
        self.assertTrue(mock.connect())

    def test_key_material_valid_pem_keys_invalid_header_auto_conversion(self):
        # Test a scenario where valid PEM keys with invalid headers which is
        # not recognized by paramiko are automatically converted in a format
        # which is recognized by paramiko
        conn_params = {"hostname": "dummy.host.org", "username": "ubuntu"}
        client = ParamikoSSHClient(**conn_params)

        # 1. RSA key type with header which is not supported by paramiko
        path = os.path.join(
            os.path.dirname(__file__),
            "fixtures",
            "misc",
            "test_rsa_non_paramiko_recognized_header.key",
        )

        with open(path) as fp:
            private_key = fp.read()

        pkey = client._get_pkey_object(key=private_key)
        self.assertTrue(pkey)
        self.assertTrue(isinstance(pkey, paramiko.RSAKey))

        # 2. DSA key type with header which is not supported by paramiko
        path = os.path.join(
            os.path.dirname(__file__),
            "fixtures",
            "misc",
            "test_dsa_non_paramiko_recognized_header.key",
        )

        with open(path) as fp:
            private_key = fp.read()

        pkey = client._get_pkey_object(key=private_key)
        self.assertTrue(pkey)
        self.assertTrue(isinstance(pkey, paramiko.DSSKey))

        # 3. ECDSA key type with header which is not supported by paramiko
        path = os.path.join(
            os.path.dirname(__file__),
            "fixtures",
            "misc",
            "test_ecdsa_non_paramiko_recognized_header.key",
        )

        with open(path) as fp:
            private_key = fp.read()

        pkey = client._get_pkey_object(key=private_key)
        self.assertTrue(pkey)
        self.assertTrue(isinstance(pkey, paramiko.ECDSAKey))

    def test_key_material_valid_pem_keys(self):
        conn_params = {"hostname": "dummy.host.org", "username": "ubuntu"}
        client = ParamikoSSHClient(**conn_params)

        # 1. RSA key type with header which is not supported by paramiko
        path = os.path.join(os.path.dirname(__file__), "fixtures", "misc", "test_rsa.key")

        with open(path) as fp:
            private_key = fp.read()

        pkey = client._get_pkey_object(key=private_key)
        self.assertTrue(pkey)
        self.assertTrue(isinstance(pkey, paramiko.RSAKey))

        # 2. DSA key type with header which is not supported by paramiko
        path = os.path.join(os.path.dirname(__file__), "fixtures", "misc", "test_dsa.key")

        with open(path) as fp:
            private_key = fp.read()

        pkey = client._get_pkey_object(key=private_key)
        self.assertTrue(pkey)
        self.assertTrue(isinstance(pkey, paramiko.DSSKey))

        # 3. ECDSA key type with header which is not supported by paramiko
        path = os.path.join(os.path.dirname(__file__), "fixtures", "misc", "test_ecdsa.key")

        with open(path) as fp:
            private_key = fp.read()

        pkey = client._get_pkey_object(key=private_key)
        self.assertTrue(pkey)
        self.assertTrue(isinstance(pkey, paramiko.ECDSAKey))

    @patch("paramiko.SSHClient", Mock)
    def test_create_with_key(self):
        conn_params = {
            "hostname": "dummy.host.org",
            "username": "ubuntu",
            "key_files": "id_rsa",
        }
        mock = ParamikoSSHClient(**conn_params)
        mock.connect()

        expected_conn = {
            "username": "ubuntu",
            "allow_agent": False,
            "hostname": "dummy.host.org",
            "look_for_keys": False,
            "key_filename": "id_rsa",
            "port": 22,
        }
        mock.client.connect.assert_called_once_with(**expected_conn)
        self.assertLogMsg("Connecting to server")

    @patch("paramiko.SSHClient", Mock)
    def test_create_with_password_and_key(self):
        conn_params = {
            "hostname": "dummy.host.org",
            "username": "ubuntu",
            "password": "ubuntu",
            "key": "id_rsa",
        }
        mock = ParamikoSSHClient(**conn_params)
        mock.connect()

        expected_conn = {
            "username": "ubuntu",
            "password": "ubuntu",
            "allow_agent": False,
            "hostname": "dummy.host.org",
            "look_for_keys": False,
            "key_filename": "id_rsa",
            "port": 22,
        }
        mock.client.connect.assert_called_once_with(**expected_conn)
        self.assertLogMsg("Connecting to server")

    @patch("paramiko.SSHClient", Mock)
    def test_create_without_credentials(self):
        """
        Initialize object with no credentials.

        Just to have better coverage, initialize the object
        without 'password' neither 'key'.
        """
        conn_params = {"hostname": "dummy.host.org", "username": "ubuntu"}
        mock = ParamikoSSHClient(**conn_params)
        mock.connect()

        expected_conn = {
            "username": "ubuntu",
            "hostname": "dummy.host.org",
            "allow_agent": True,
            "look_for_keys": True,
            "port": 22,
        }
        mock.client.connect.assert_called_once_with(**expected_conn)

    @patch.object(ParamikoSSHClient, "_consume_stdout", MagicMock(return_value=StringIO("")))
    @patch.object(ParamikoSSHClient, "_consume_stderr", MagicMock(return_value=StringIO("")))
    def test_basic_usage_absolute_path(self):
        """
        Basic execution.
        """
        mock = self.ssh_cli
        # script to execute
        sd = "/root/random_script.sh"

        # Connect behavior
        mock.connect()
        mock_cli = mock.client  # The actual mocked object: SSHClient

        expected_conn = {
            "username": "ubuntu",
            "key_filename": "~/.ssh/ubuntu_ssh",
            "allow_agent": False,
            "hostname": "dummy.host.org",
            "look_for_keys": False,
            "timeout": "600",
            "port": 8822,
        }
        mock_cli.connect.assert_called_once_with(**expected_conn)

        mock.put(sd)
        # Make assertions over 'put' method
        mock_cli.open_sftp().chdir.assert_called_with("root")
        mock_cli.open_sftp().file.assert_called_once_with("random_script.sh", mode="w")

        mock.run(sd)

        # Make assertions over 'run' method
        mock_cli.get_transport().open_session().exec_command.assert_called_once_with(sd)
        self.assertLogMsg("Executing command (cmd=/root/random_script.sh)")
        self.assertLogMsg("Command finished")

        mock.close()

    def test_delete_script(self):
        """
        Provide a basic test with 'delete' action.
        """
        mock = self.ssh_cli
        # script to execute
        sd = "/root/random_script.sh"

        mock.connect()

        mock.delete(sd)
        # Make assertions over the 'delete' method
        mock.client.open_sftp().unlink.assert_called_with(sd)
        self.assertLogMsg("Deleting file")

        mock.close()
        self.assertLogMsg("Closing server connection")

    def assertLogMsg(self, expected_msg):
        with open(self.tmp_file) as fp:
            content = fp.read()

        self.assertTrue(content.find(expected_msg) != -1)

    def test_consume_stdout(self):
        conn_params = {"hostname": "dummy.host.org", "username": "ubuntu"}
        client = ParamikoSSHClient(**conn_params)
        client.CHUNK_SIZE = 1024

        chan = Mock()
        chan.recv_ready.side_effect = [True, True, False]
        chan.recv.side_effect = ["123", "456"]

        stdout = client._consume_stdout(chan).getvalue()
        self.assertEqual(u("123456"), stdout)
        self.assertEqual(len(stdout), 6)

        conn_params = {"hostname": "dummy.host.org", "username": "ubuntu"}
        client = ParamikoSSHClient(**conn_params)
        client.CHUNK_SIZE = 1024

        chan = Mock()
        chan.recv_ready.side_effect = [True, True, False]
        chan.recv.side_effect = ["987", "6543210"]

        stdout = client._consume_stdout(chan).getvalue()
        self.assertEqual(u("9876543210"), stdout)
        self.assertEqual(len(stdout), 10)

    def test_consume_stderr(self):
        conn_params = {"hostname": "dummy.host.org", "username": "ubuntu"}
        client = ParamikoSSHClient(**conn_params)
        client.CHUNK_SIZE = 1024

        chan = Mock()
        chan.recv_stderr_ready.side_effect = [True, True, False]
        chan.recv_stderr.side_effect = ["123", "456"]

        stderr = client._consume_stderr(chan).getvalue()
        self.assertEqual(u("123456"), stderr)
        self.assertEqual(len(stderr), 6)

        conn_params = {"hostname": "dummy.host.org", "username": "ubuntu"}
        client = ParamikoSSHClient(**conn_params)
        client.CHUNK_SIZE = 1024

        chan = Mock()
        chan.recv_stderr_ready.side_effect = [True, True, False]
        chan.recv_stderr.side_effect = ["987", "6543210"]

        stderr = client._consume_stderr(chan).getvalue()
        self.assertEqual(u("9876543210"), stderr)
        self.assertEqual(len(stderr), 10)

    def test_consume_stdout_chunk_contains_part_of_multi_byte_utf8_character(self):
        conn_params = {"hostname": "dummy.host.org", "username": "ubuntu"}
        client = ParamikoSSHClient(**conn_params)
        client.CHUNK_SIZE = 1

        chan = Mock()
        chan.recv_ready.side_effect = [True, True, True, True, False]
        chan.recv.side_effect = ["\xF0", "\x90", "\x8D", "\x88"]

        stdout = client._consume_stdout(chan).getvalue()
        self.assertEqual("ð\x90\x8d\x88", stdout)
        self.assertEqual(len(stdout), 4)

    def test_consume_stderr_chunk_contains_part_of_multi_byte_utf8_character(self):
        conn_params = {"hostname": "dummy.host.org", "username": "ubuntu"}
        client = ParamikoSSHClient(**conn_params)
        client.CHUNK_SIZE = 1

        chan = Mock()
        chan.recv_stderr_ready.side_effect = [True, True, True, True, False]
        chan.recv_stderr.side_effect = ["\xF0", "\x90", "\x8D", "\x88"]

        stderr = client._consume_stderr(chan).getvalue()
        self.assertEqual("ð\x90\x8d\x88", stderr)
        self.assertEqual(len(stderr), 4)

    def test_consume_stdout_chunk_contains_non_utf8_character(self):
        conn_params = {"hostname": "dummy.host.org", "username": "ubuntu"}
        client = ParamikoSSHClient(**conn_params)
        client.CHUNK_SIZE = 1

        chan = Mock()
        chan.recv_ready.side_effect = [True, True, True, False]
        chan.recv.side_effect = ["🤦".encode("utf-32"), "a", "b"]

        stdout = client._consume_stdout(chan).getvalue()
        if sys.byteorder == "little":
            self.assertEqual("\x00\x00&\x01\x00ab", stdout)
        else:
            self.assertEqual("\x00\x00\x00\x01&ab", stdout)
        self.assertEqual(len(stdout), 7)

    def test_consume_stderr_chunk_contains_non_utf8_character(self):
        conn_params = {"hostname": "dummy.host.org", "username": "ubuntu"}
        client = ParamikoSSHClient(**conn_params)
        client.CHUNK_SIZE = 1

        chan = Mock()
        chan.recv_stderr_ready.side_effect = [True, True, True, False]
        chan.recv_stderr.side_effect = ["🤦".encode("utf-32"), "a", "b"]

        stderr = client._consume_stderr(chan).getvalue()
        if sys.byteorder == "little":
            self.assertEqual("\x00\x00&\x01\x00ab", stderr)
        else:
            self.assertEqual("\x00\x00\x00\x01&ab", stderr)
        self.assertEqual(len(stderr), 7)

    def test_keep_alive_and_compression(self):
        conn_params = {"hostname": "dummy.host.org", "username": "ubuntu"}
        client = ParamikoSSHClient(**conn_params)

        mock_transport = Mock()
        client.client.get_transport = Mock(return_value=mock_transport)

        transport = client._get_transport()
        self.assertEqual(transport.set_keepalive.call_count, 0)
        self.assertEqual(transport.use_compression.call_count, 0)

        conn_params = {
            "hostname": "dummy.host.org",
            "username": "ubuntu",
            "keep_alive": 15,
            "use_compression": True,
        }
        client = ParamikoSSHClient(**conn_params)

        mock_transport = Mock()
        client.client.get_transport = Mock(return_value=mock_transport)

        transport = client._get_transport()
        self.assertEqual(transport.set_keepalive.call_count, 1)
        self.assertEqual(transport.use_compression.call_count, 1)

    def test_put_absolute_path(self):
        conn_params = {"hostname": "dummy.host.org", "username": "ubuntu"}
        client = ParamikoSSHClient(**conn_params)

        mock_client = Mock()
        mock_sftp_client = Mock()
        mock_transport = Mock()

        mock_client.get_transport.return_value = mock_transport
        mock_sftp_client.getcwd.return_value = "/mock/cwd"
        client.client = mock_client
        client.sftp_client = mock_sftp_client

        result = client.put(path="/test/remote/path.txt", contents="foo bar", chmod=455, mode="w")
        self.assertEqual(result, "/test/remote/path.txt")

        calls = [call("/"), call("test"), call("remote")]
        mock_sftp_client.chdir.assert_has_calls(calls, any_order=False)

        calls = [
            call("path.txt", mode="w"),
            call().write("foo bar"),
            call().chmod(455),
            call().close(),
        ]
        mock_sftp_client.file.assert_has_calls(calls, any_order=False)

    def test_put_absolute_path_windows(self):
        conn_params = {"hostname": "dummy.host.org", "username": "ubuntu"}
        client = ParamikoSSHClient(**conn_params)

        mock_client = Mock()
        mock_sftp_client = Mock()
        mock_transport = Mock()

        mock_client.get_transport.return_value = mock_transport
        mock_sftp_client.getcwd.return_value = "C:\\Administrator"
        client.client = mock_client
        client.sftp_client = mock_sftp_client

        result = client.put(path="C:\\users\\user1\\1.txt", contents="foo bar", chmod=455, mode="w")
        self.assertEqual(result, "C:\\users\\user1\\1.txt")

        result = client.put(path="\\users\\user1\\1.txt", contents="foo bar", chmod=455, mode="w")
        self.assertEqual(result, "\\users\\user1\\1.txt")

        result = client.put(path="1.txt", contents="foo bar", chmod=455, mode="w")
        self.assertEqual(result, "C:\\Administrator\\1.txt")

        mock_client.get_transport.return_value = mock_transport
        mock_sftp_client.getcwd.return_value = "/C:\\User1"
        client.client = mock_client
        client.sftp_client = mock_sftp_client

        result = client.put(path="1.txt", contents="foo bar", chmod=455, mode="w")
        self.assertEqual(result, "C:\\User1\\1.txt")

    def test_put_relative_path(self):
        conn_params = {"hostname": "dummy.host.org", "username": "ubuntu"}
        client = ParamikoSSHClient(**conn_params)

        mock_client = Mock()
        mock_sftp_client = Mock()
        mock_transport = Mock()

        mock_client.get_transport.return_value = mock_transport
        mock_sftp_client.getcwd.return_value = "/mock/cwd"
        client.client = mock_client
        client.sftp_client = mock_sftp_client

        result = client.put(path="path2.txt", contents="foo bar 2", chmod=466, mode="a")
        self.assertEqual(result, "/mock/cwd/path2.txt")

        calls = [call(".")]
        mock_sftp_client.chdir.assert_has_calls(calls, any_order=False)

        calls = [
            call("path2.txt", mode="a"),
            call().write("foo bar 2"),
            call().chmod(466),
            call().close(),
        ]
        mock_sftp_client.file.assert_has_calls(calls, any_order=False)

    def test_putfo_absolute_path(self):
        conn_params = {"hostname": "dummy.host.org", "username": "ubuntu"}
        client = ParamikoSSHClient(**conn_params)

        mock_client = Mock()
        mock_sftp_client = Mock()
        mock_transport = Mock()

        mock_client.get_transport.return_value = mock_transport
        mock_sftp_client.getcwd.return_value = "/mock/cwd"
        client.client = mock_client
        client.sftp_client = mock_sftp_client

        mock_fo = StringIO("mock stream data 1")

        result = client.putfo(path="/test/remote/path.txt", fo=mock_fo, chmod=455)
        self.assertEqual(result, "/test/remote/path.txt")

        calls = [call("/"), call("test"), call("remote")]
        mock_sftp_client.chdir.assert_has_calls(calls, any_order=False)

        mock_sftp_client.putfo.assert_called_once_with(mock_fo, "/test/remote/path.txt")

        calls = [call("path.txt"), call().chmod(455), call().close()]
        mock_sftp_client.file.assert_has_calls(calls, any_order=False)

    def test_putfo_relative_path(self):
        conn_params = {"hostname": "dummy.host.org", "username": "ubuntu"}
        client = ParamikoSSHClient(**conn_params)

        mock_client = Mock()
        mock_sftp_client = Mock()
        mock_transport = Mock()

        mock_client.get_transport.return_value = mock_transport
        mock_sftp_client.getcwd.return_value = "/mock/cwd"
        client.client = mock_client
        client.sftp_client = mock_sftp_client

        mock_fo = StringIO("mock stream data 2")

        result = client.putfo(path="path2.txt", fo=mock_fo, chmod=466)
        self.assertEqual(result, "/mock/cwd/path2.txt")

        calls = [call(".")]
        mock_sftp_client.chdir.assert_has_calls(calls, any_order=False)

        mock_sftp_client.putfo.assert_called_once_with(mock_fo, "path2.txt")

        calls = [call("path2.txt"), call().chmod(466), call().close()]
        mock_sftp_client.file.assert_has_calls(calls, any_order=False)

    def test_get_sftp_client(self):
        conn_params = {"hostname": "dummy.host.org", "username": "ubuntu"}
        client = ParamikoSSHClient(**conn_params)

        # 1. sftp connection is not opened yet, new one should be opened
        mock_client = Mock()
        mock_sft_client = Mock()
        mock_client.open_sftp.return_value = mock_sft_client
        client.client = mock_client

        self.assertEqual(mock_client.open_sftp.call_count, 0)
        self.assertEqual(client._get_sftp_client(), mock_sft_client)
        self.assertEqual(mock_client.open_sftp.call_count, 1)

        # 2. existing sftp connection which is already opened is re-used
        mock_client = Mock()
        mock_sft_client = Mock()
        client.client = mock_client
        client.sftp_client = mock_sft_client

        self.assertEqual(mock_client.open_sftp.call_count, 0)
        self.assertEqual(client._get_sftp_client(), mock_sft_client)
        self.assertEqual(mock_client.open_sftp.call_count, 0)

        # 3. existing connection is already opened, but it throws
        # "socket closed" error, we should establish a new one
        mock_client = Mock()
        mock_sftp_client = Mock()

        client.client = mock_client
        client.sftp_client = mock_sftp_client

        mock_sftp_client.listdir.side_effect = OSError("Socket is closed")

        self.assertEqual(mock_client.open_sftp.call_count, 0)
        sftp_client = client._get_sftp_client()
        self.assertTrue(sftp_client != mock_sft_client)
        self.assertTrue(sftp_client)
        self.assertTrue(client._get_sftp_client())
        self.assertEqual(mock_client.open_sftp.call_count, 1)

        # 4. fatal exceptions should be propagated
        mock_client = Mock()
        mock_sftp_client = Mock()

        client.client = mock_client
        client.sftp_client = mock_sftp_client

        mock_sftp_client.listdir.side_effect = Exception("Fatal exception")

        self.assertEqual(mock_client.open_sftp.call_count, 0)
        self.assertRaisesRegex(Exception, "Fatal exception", client._get_sftp_client)


class ShellOutSSHClientTests(LibcloudTestCase):
    def test_password_auth_not_supported(self):
        try:
            ShellOutSSHClient(hostname="localhost", username="foo", password="bar")
        except ValueError as e:
            msg = str(e)
            self.assertTrue("ShellOutSSHClient only supports key auth" in msg)
        else:
            self.fail("Exception was not thrown")

    def test_ssh_executable_not_available(self):
        class MockChild:
            returncode = 127

            def communicate(*args, **kwargs):
                pass

        def mock_popen(*args, **kwargs):
            return MockChild()

        with patch("subprocess.Popen", mock_popen):
            try:
                ShellOutSSHClient(hostname="localhost", username="foo")
            except ValueError as e:
                msg = str(e)
                self.assertTrue("ssh client is not available" in msg)
            else:
                self.fail("Exception was not thrown")

    def test_connect_success(self):
        client = ShellOutSSHClient(hostname="localhost", username="root")
        self.assertTrue(client.connect())

    def test_close_success(self):
        client = ShellOutSSHClient(hostname="localhost", username="root")
        self.assertTrue(client.close())

    def test_get_base_ssh_command(self):
        client1 = ShellOutSSHClient(hostname="localhost", username="root")
        client2 = ShellOutSSHClient(hostname="localhost", username="root", key="/home/my.key")
        client3 = ShellOutSSHClient(
            hostname="localhost", username="root", key="/home/my.key", timeout=5
        )

        cmd1 = client1._get_base_ssh_command()
        cmd2 = client2._get_base_ssh_command()
        cmd3 = client3._get_base_ssh_command()

        self.assertEqual(cmd1, ["ssh", "root@localhost"])
        self.assertEqual(cmd2, ["ssh", "-i", "/home/my.key", "root@localhost"])
        self.assertEqual(
            cmd3, ["ssh", "-i", "/home/my.key", "-oConnectTimeout=5", "root@localhost"]
        )


if __name__ == "__main__":
    sys.exit(unittest.main())



# Copyright 2002-2009, Distributed Systems Architecture Group, Universidad
# Complutense de Madrid (dsa-research.org)
#
# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
OpenNebula.org test suite.
"""

__docformat__ = "epytext"

import sys
import unittest

import libcloud.compute.drivers.opennebula
from libcloud.test import MockHttp
from libcloud.utils.py3 import httplib
from libcloud.compute.base import Node, NodeSize, NodeImage, NodeState
from libcloud.test.secrets import OPENNEBULA_PARAMS
from libcloud.test.file_fixtures import ComputeFileFixtures
from libcloud.compute.drivers.opennebula import (
    ACTION,
    OpenNebulaNetwork,
    OpenNebulaNodeSize,
    OpenNebulaNodeDriver,
)

libcloud.compute.drivers.opennebula.API_HOST = "dummy"


class OpenNebula_1_4_Tests(unittest.TestCase):

    """
    OpenNebula.org test suite for OpenNebula v1.4.
    """

    def setUp(self):
        """
        Setup test environment.
        """
        OpenNebulaNodeDriver.connectionCls.conn_class = OpenNebula_1_4_MockHttp
        self.driver = OpenNebulaNodeDriver(*OPENNEBULA_PARAMS + ("1.4",), host="dummy")

    def test_create_node(self):
        """
        Test create_node functionality.
        """
        image = NodeImage(id=5, name="Ubuntu 9.04 LAMP", driver=self.driver)
        size = NodeSize(
            id=1,
            name="small",
            ram=None,
            disk=None,
            bandwidth=None,
            price=None,
            driver=self.driver,
        )
        networks = list()
        networks.append(
            OpenNebulaNetwork(
                id=5,
                name="Network 5",
                address="192.168.0.0",
                size=256,
                driver=self.driver,
            )
        )
        networks.append(
            OpenNebulaNetwork(
                id=15,
                name="Network 15",
                address="192.168.1.0",
                size=256,
                driver=self.driver,
            )
        )

        node = self.driver.create_node(name="Compute 5", image=image, size=size, networks=networks)

        self.assertEqual(node.id, "5")
        self.assertEqual(node.name, "Compute 5")
        self.assertEqual(node.state, OpenNebulaNodeDriver.NODE_STATE_MAP["ACTIVE"])
        self.assertIsNone(node.public_ips[0].name)
        self.assertEqual(node.public_ips[0].id, "5")
        self.assertEqual(node.public_ips[0].address, "192.168.0.1")
        self.assertEqual(node.public_ips[0].size, 1)
        self.assertIsNone(node.public_ips[1].name)
        self.assertEqual(node.public_ips[1].id, "15")
        self.assertEqual(node.public_ips[1].address, "192.168.1.1")
        self.assertEqual(node.public_ips[1].size, 1)
        self.assertEqual(node.private_ips, [])
        self.assertEqual(node.image.id, "5")
        self.assertEqual(node.image.extra["dev"], "sda1")

    def test_destroy_node(self):
        """
        Test destroy_node functionality.
        """
        node = Node(5, None, None, None, None, self.driver)
        ret = self.driver.destroy_node(node)
        self.assertTrue(ret)

    def test_list_nodes(self):
        """
        Test list_nodes functionality.
        """
        nodes = self.driver.list_nodes()

        self.assertEqual(len(nodes), 3)
        node = nodes[0]
        self.assertEqual(node.id, "5")
        self.assertEqual(node.name, "Compute 5")
        self.assertEqual(node.state, OpenNebulaNodeDriver.NODE_STATE_MAP["ACTIVE"])
        self.assertEqual(node.public_ips[0].id, "5")
        self.assertIsNone(node.public_ips[0].name)
        self.assertEqual(node.public_ips[0].address, "192.168.0.1")
        self.assertEqual(node.public_ips[0].size, 1)
        self.assertEqual(node.public_ips[1].id, "15")
        self.assertIsNone(node.public_ips[1].name)
        self.assertEqual(node.public_ips[1].address, "192.168.1.1")
        self.assertEqual(node.public_ips[1].size, 1)
        self.assertEqual(node.private_ips, [])
        self.assertEqual(node.image.id, "5")
        self.assertEqual(node.image.extra["dev"], "sda1")
        node = nodes[1]
        self.assertEqual(node.id, "15")
        self.assertEqual(node.name, "Compute 15")
        self.assertEqual(node.state, OpenNebulaNodeDriver.NODE_STATE_MAP["ACTIVE"])
        self.assertEqual(node.public_ips[0].id, "5")
        self.assertIsNone(node.public_ips[0].name)
        self.assertEqual(node.public_ips[0].address, "192.168.0.2")
        self.assertEqual(node.public_ips[0].size, 1)
        self.assertEqual(node.public_ips[1].id, "15")
        self.assertIsNone(node.public_ips[1].name)
        self.assertEqual(node.public_ips[1].address, "192.168.1.2")
        self.assertEqual(node.public_ips[1].size, 1)
        self.assertEqual(node.private_ips, [])
        self.assertEqual(node.image.id, "15")
        self.assertEqual(node.image.extra["dev"], "sda1")
        node = nodes[2]
        self.assertEqual(node.id, "25")
        self.assertEqual(node.name, "Compute 25")
        self.assertEqual(node.state, NodeState.UNKNOWN)
        self.assertEqual(node.public_ips[0].id, "5")
        self.assertIsNone(node.public_ips[0].name)
        self.assertEqual(node.public_ips[0].address, "192.168.0.3")
        self.assertEqual(node.public_ips[0].size, 1)
        self.assertEqual(node.public_ips[1].id, "15")
        self.assertIsNone(node.public_ips[1].name)
        self.assertEqual(node.public_ips[1].address, "192.168.1.3")
        self.assertEqual(node.public_ips[1].size, 1)
        self.assertEqual(node.private_ips, [])
        self.assertIsNone(node.image)

    def test_list_images(self):
        """
        Test list_images functionality.
        """
        images = self.driver.list_images()

        self.assertEqual(len(images), 2)
        image = images[0]
        self.assertEqual(image.id, "5")
        self.assertEqual(image.name, "Ubuntu 9.04 LAMP")
        self.assertEqual(image.extra["size"], "2048")
        self.assertEqual(image.extra["url"], "file:///images/ubuntu/jaunty.img")
        image = images[1]
        self.assertEqual(image.id, "15")
        self.assertEqual(image.name, "Ubuntu 9.04 LAMP")
        self.assertEqual(image.extra["size"], "2048")
        self.assertEqual(image.extra["url"], "file:///images/ubuntu/jaunty.img")

    def test_list_sizes(self):
        """
        Test list_sizes functionality.
        """
        sizes = self.driver.list_sizes()

        self.assertEqual(len(sizes), 3)
        size = sizes[0]
        self.assertEqual(size.id, "1")
        self.assertEqual(size.name, "small")
        self.assertIsNone(size.ram)
        self.assertIsNone(size.disk)
        self.assertIsNone(size.bandwidth)
        self.assertIsNone(size.price)
        size = sizes[1]
        self.assertEqual(size.id, "2")
        self.assertEqual(size.name, "medium")
        self.assertIsNone(size.ram)
        self.assertIsNone(size.disk)
        self.assertIsNone(size.bandwidth)
        self.assertIsNone(size.price)
        size = sizes[2]
        self.assertEqual(size.id, "3")
        self.assertEqual(size.name, "large")
        self.assertIsNone(size.ram)
        self.assertIsNone(size.disk)
        self.assertIsNone(size.bandwidth)
        self.assertIsNone(size.price)

    def test_list_locations(self):
        """
        Test list_locations functionality.
        """
        locations = self.driver.list_locations()

        self.assertEqual(len(locations), 1)
        location = locations[0]
        self.assertEqual(location.id, "0")
        self.assertEqual(location.name, "")
        self.assertEqual(location.country, "")

    def test_ex_list_networks(self):
        """
        Test ex_list_networks functionality.
        """
        networks = self.driver.ex_list_networks()

        self.assertEqual(len(networks), 2)
        network = networks[0]
        self.assertEqual(network.id, "5")
        self.assertEqual(network.name, "Network 5")
        self.assertEqual(network.address, "192.168.0.0")
        self.assertEqual(network.size, "256")
        network = networks[1]
        self.assertEqual(network.id, "15")
        self.assertEqual(network.name, "Network 15")
        self.assertEqual(network.address, "192.168.1.0")
        self.assertEqual(network.size, "256")

    def test_ex_node_action(self):
        """
        Test ex_node_action functionality.
        """
        node = Node(5, None, None, None, None, self.driver)
        ret = self.driver.ex_node_action(node, ACTION.STOP)
        self.assertTrue(ret)


class OpenNebula_2_0_Tests(unittest.TestCase):

    """
    OpenNebula.org test suite for OpenNebula v2.0 through v2.2.
    """

    def setUp(self):
        """
        Setup test environment.
        """
        OpenNebulaNodeDriver.connectionCls.conn_class = OpenNebula_2_0_MockHttp
        self.driver = OpenNebulaNodeDriver(*OPENNEBULA_PARAMS + ("2.0",), host="dummy")

    def test_create_node(self):
        """
        Test create_node functionality.
        """
        image = NodeImage(id=5, name="Ubuntu 9.04 LAMP", driver=self.driver)
        size = OpenNebulaNodeSize(
            id=1,
            name="small",
            ram=1024,
            cpu=1,
            disk=None,
            bandwidth=None,
            price=None,
            driver=self.driver,
        )
        networks = list()
        networks.append(
            OpenNebulaNetwork(
                id=5,
                name="Network 5",
                address="192.168.0.0",
                size=256,
                driver=self.driver,
            )
        )
        networks.append(
            OpenNebulaNetwork(
                id=15,
                name="Network 15",
                address="192.168.1.0",
                size=256,
                driver=self.driver,
            )
        )
        context = {"hostname": "compute-5"}

        node = self.driver.create_node(
            name="Compute 5", image=image, size=size, networks=networks, context=context
        )

        self.assertEqual(node.id, "5")
        self.assertEqual(node.name, "Compute 5")
        self.assertEqual(node.state, OpenNebulaNodeDriver.NODE_STATE_MAP["ACTIVE"])
        self.assertEqual(node.public_ips[0].id, "5")
        self.assertEqual(node.public_ips[0].name, "Network 5")
        self.assertEqual(node.public_ips[0].address, "192.168.0.1")
        self.assertEqual(node.public_ips[0].size, 1)
        self.assertEqual(node.public_ips[0].extra["mac"], "02:00:c0:a8:00:01")
        self.assertEqual(node.public_ips[1].id, "15")
        self.assertEqual(node.public_ips[1].name, "Network 15")
        self.assertEqual(node.public_ips[1].address, "192.168.1.1")
        self.assertEqual(node.public_ips[1].size, 1)
        self.assertEqual(node.public_ips[1].extra["mac"], "02:00:c0:a8:01:01")
        self.assertEqual(node.private_ips, [])
        self.assertTrue(len([s for s in self.driver.list_sizes() if s.id == node.size.id]) == 1)
        self.assertEqual(node.image.id, "5")
        self.assertEqual(node.image.name, "Ubuntu 9.04 LAMP")
        self.assertEqual(node.image.extra["type"], "DISK")
        self.assertEqual(node.image.extra["target"], "hda")
        context = node.extra["context"]
        self.assertEqual(context["hostname"], "compute-5")

    def test_destroy_node(self):
        """
        Test destroy_node functionality.
        """
        node = Node(5, None, None, None, None, self.driver)
        ret = self.driver.destroy_node(node)
        self.assertTrue(ret)

    def test_list_nodes(self):
        """
        Test list_nodes functionality.
        """
        nodes = self.driver.list_nodes()

        self.assertEqual(len(nodes), 3)
        node = nodes[0]
        self.assertEqual(node.id, "5")
        self.assertEqual(node.name, "Compute 5")
        self.assertEqual(node.state, OpenNebulaNodeDriver.NODE_STATE_MAP["ACTIVE"])
        self.assertEqual(node.public_ips[0].id, "5")
        self.assertEqual(node.public_ips[0].name, "Network 5")
        self.assertEqual(node.public_ips[0].address, "192.168.0.1")
        self.assertEqual(node.public_ips[0].size, 1)
        self.assertEqual(node.public_ips[0].extra["mac"], "02:00:c0:a8:00:01")
        self.assertEqual(node.public_ips[1].id, "15")
        self.assertEqual(node.public_ips[1].name, "Network 15")
        self.assertEqual(node.public_ips[1].address, "192.168.1.1")
        self.assertEqual(node.public_ips[1].size, 1)
        self.assertEqual(node.public_ips[1].extra["mac"], "02:00:c0:a8:01:01")
        self.assertEqual(node.private_ips, [])
        self.assertTrue(
            len([size for size in self.driver.list_sizes() if size.id == node.size.id]) == 1
        )
        self.assertEqual(node.size.id, "1")
        self.assertEqual(node.size.name, "small")
        self.assertEqual(node.size.ram, 1024)
        self.assertTrue(node.size.cpu is None or isinstance(node.size.cpu, int))
        self.assertTrue(node.size.vcpu is None or isinstance(node.size.vcpu, int))
        self.assertEqual(node.size.cpu, 1)
        self.assertIsNone(node.size.vcpu)
        self.assertIsNone(node.size.disk)
        self.assertIsNone(node.size.bandwidth)
        self.assertIsNone(node.size.price)
        self.assertTrue(
            len([image for image in self.driver.list_images() if image.id == node.image.id]) == 1
        )
        self.assertEqual(node.image.id, "5")
        self.assertEqual(node.image.name, "Ubuntu 9.04 LAMP")
        self.assertEqual(node.image.extra["type"], "DISK")
        self.assertEqual(node.image.extra["target"], "hda")
        context = node.extra["context"]
        self.assertEqual(context["hostname"], "compute-5")
        node = nodes[1]
        self.assertEqual(node.id, "15")
        self.assertEqual(node.name, "Compute 15")
        self.assertEqual(node.state, OpenNebulaNodeDriver.NODE_STATE_MAP["ACTIVE"])
        self.assertEqual(node.public_ips[0].id, "5")
        self.assertEqual(node.public_ips[0].name, "Network 5")
        self.assertEqual(node.public_ips[0].address, "192.168.0.2")
        self.assertEqual(node.public_ips[0].size, 1)
        self.assertEqual(node.public_ips[0].extra["mac"], "02:00:c0:a8:00:02")
        self.assertEqual(node.public_ips[1].id, "15")
        self.assertEqual(node.public_ips[1].name, "Network 15")
        self.assertEqual(node.public_ips[1].address, "192.168.1.2")
        self.assertEqual(node.public_ips[1].size, 1)
        self.assertEqual(node.public_ips[1].extra["mac"], "02:00:c0:a8:01:02")
        self.assertEqual(node.private_ips, [])
        self.assertTrue(
            len([size for size in self.driver.list_sizes() if size.id == node.size.id]) == 1
        )
        self.assertEqual(node.size.id, "1")
        self.assertEqual(node.size.name, "small")
        self.assertEqual(node.size.ram, 1024)
        self.assertTrue(node.size.cpu is None or isinstance(node.size.cpu, int))
        self.assertTrue(node.size.vcpu is None or isinstance(node.size.vcpu, int))
        self.assertEqual(node.size.cpu, 1)
        self.assertIsNone(node.size.vcpu)
        self.assertIsNone(node.size.disk)
        self.assertIsNone(node.size.bandwidth)
        self.assertIsNone(node.size.price)
        self.assertTrue(
            len([image for image in self.driver.list_images() if image.id == node.image.id]) == 1
        )
        self.assertEqual(node.image.id, "15")
        self.assertEqual(node.image.name, "Ubuntu 9.04 LAMP")
        self.assertEqual(node.image.extra["type"], "DISK")
        self.assertEqual(node.image.extra["target"], "hda")
        context = node.extra["context"]
        self.assertEqual(context["hostname"], "compute-15")
        node = nodes[2]
        self.assertEqual(node.id, "25")
        self.assertEqual(node.name, "Compute 25")
        self.assertEqual(node.state, NodeState.UNKNOWN)
        self.assertEqual(node.public_ips[0].id, "5")
        self.assertEqual(node.public_ips[0].name, "Network 5")
        self.assertEqual(node.public_ips[0].address, "192.168.0.3")
        self.assertEqual(node.public_ips[0].size, 1)
        self.assertEqual(node.public_ips[0].extra["mac"], "02:00:c0:a8:00:03")
        self.assertEqual(node.public_ips[1].id, "15")
        self.assertEqual(node.public_ips[1].name, "Network 15")
        self.assertEqual(node.public_ips[1].address, "192.168.1.3")
        self.assertEqual(node.public_ips[1].size, 1)
        self.assertEqual(node.public_ips[1].extra["mac"], "02:00:c0:a8:01:03")
        self.assertEqual(node.private_ips, [])
        self.assertIsNone(node.size)
        self.assertIsNone(node.image)
        context = node.extra["context"]
        self.assertEqual(context, {})

    def test_list_images(self):
        """
        Test list_images functionality.
        """
        images = self.driver.list_images()

        self.assertEqual(len(images), 2)
        image = images[0]
        self.assertEqual(image.id, "5")
        self.assertEqual(image.name, "Ubuntu 9.04 LAMP")
        self.assertEqual(image.extra["description"], "Ubuntu 9.04 LAMP Description")
        self.assertEqual(image.extra["type"], "OS")
        self.assertEqual(image.extra["size"], "2048")
        image = images[1]
        self.assertEqual(image.id, "15")
        self.assertEqual(image.name, "Ubuntu 9.04 LAMP")
        self.assertEqual(image.extra["description"], "Ubuntu 9.04 LAMP Description")
        self.assertEqual(image.extra["type"], "OS")
        self.assertEqual(image.extra["size"], "2048")

    def test_list_sizes(self):
        """
        Test list_sizes functionality.
        """
        sizes = self.driver.list_sizes()

        self.assertEqual(len(sizes), 4)
        size = sizes[0]
        self.assertEqual(size.id, "1")
        self.assertEqual(size.name, "small")
        self.assertEqual(size.ram, 1024)
        self.assertTrue(size.cpu is None or isinstance(size.cpu, int))
        self.assertTrue(size.vcpu is None or isinstance(size.vcpu, int))
        self.assertEqual(size.cpu, 1)
        self.assertIsNone(size.vcpu)
        self.assertIsNone(size.disk)
        self.assertIsNone(size.bandwidth)
        self.assertIsNone(size.price)
        size = sizes[1]
        self.assertEqual(size.id, "2")
        self.assertEqual(size.name, "medium")
        self.assertEqual(size.ram, 4096)
        self.assertTrue(size.cpu is None or isinstance(size.cpu, int))
        self.assertTrue(size.vcpu is None or isinstance(size.vcpu, int))
        self.assertEqual(size.cpu, 4)
        self.assertIsNone(size.vcpu)
        self.assertIsNone(size.disk)
        self.assertIsNone(size.bandwidth)
        self.assertIsNone(size.price)
        size = sizes[2]
        self.assertEqual(size.id, "3")
        self.assertEqual(size.name, "large")
        self.assertEqual(size.ram, 8192)
        self.assertTrue(size.cpu is None or isinstance(size.cpu, int))
        self.assertTrue(size.vcpu is None or isinstance(size.vcpu, int))
        self.assertEqual(size.cpu, 8)
        self.assertIsNone(size.vcpu)
        self.assertIsNone(size.disk)
        self.assertIsNone(size.bandwidth)
        self.assertIsNone(size.price)
        size = sizes[3]
        self.assertEqual(size.id, "4")
        self.assertEqual(size.name, "custom")
        self.assertEqual(size.ram, 0)
        self.assertEqual(size.cpu, 0)
        self.assertIsNone(size.vcpu)
        self.assertIsNone(size.disk)
        self.assertIsNone(size.bandwidth)
        self.assertIsNone(size.price)

    def test_list_locations(self):
        """
        Test list_locations functionality.
        """
        locations = self.driver.list_locations()

        self.assertEqual(len(locations), 1)
        location = locations[0]
        self.assertEqual(location.id, "0")
        self.assertEqual(location.name, "")
        self.assertEqual(location.country, "")

    def test_ex_list_networks(self):
        """
        Test ex_list_networks functionality.
        """
        networks = self.driver.ex_list_networks()

        self.assertEqual(len(networks), 2)
        network = networks[0]
        self.assertEqual(network.id, "5")
        self.assertEqual(network.name, "Network 5")
        self.assertEqual(network.address, "192.168.0.0")
        self.assertEqual(network.size, "256")
        network = networks[1]
        self.assertEqual(network.id, "15")
        self.assertEqual(network.name, "Network 15")
        self.assertEqual(network.address, "192.168.1.0")
        self.assertEqual(network.size, "256")


class OpenNebula_3_0_Tests(unittest.TestCase):

    """
    OpenNebula.org test suite for OpenNebula v3.0.
    """

    def setUp(self):
        """
        Setup test environment.
        """
        OpenNebulaNodeDriver.connectionCls.conn_class = OpenNebula_3_0_MockHttp
        self.driver = OpenNebulaNodeDriver(*OPENNEBULA_PARAMS + ("3.0",), host="dummy")

    def test_ex_list_networks(self):
        """
        Test ex_list_networks functionality.
        """
        networks = self.driver.ex_list_networks()

        self.assertEqual(len(networks), 2)
        network = networks[0]
        self.assertEqual(network.id, "5")
        self.assertEqual(network.name, "Network 5")
        self.assertEqual(network.address, "192.168.0.0")
        self.assertEqual(network.size, "256")
        self.assertEqual(network.extra["public"], "YES")
        network = networks[1]
        self.assertEqual(network.id, "15")
        self.assertEqual(network.name, "Network 15")
        self.assertEqual(network.address, "192.168.1.0")
        self.assertEqual(network.size, "256")
        self.assertEqual(network.extra["public"], "NO")

    def test_ex_node_set_save_name(self):
        """
        Test ex_node_action functionality.
        """
        image = NodeImage(id=5, name="Ubuntu 9.04 LAMP", driver=self.driver)
        node = Node(5, None, None, None, None, self.driver, image=image)
        ret = self.driver.ex_node_set_save_name(node, "test")
        self.assertTrue(ret)


class OpenNebula_3_2_Tests(unittest.TestCase):

    """
    OpenNebula.org test suite for OpenNebula v3.2.
    """

    def setUp(self):
        """
        Setup test environment.
        """
        OpenNebulaNodeDriver.connectionCls.conn_class = OpenNebula_3_2_MockHttp
        self.driver = OpenNebulaNodeDriver(*OPENNEBULA_PARAMS + ("3.2",), host="dummy")

    def test_reboot_node(self):
        """
        Test reboot_node functionality.
        """
        image = NodeImage(id=5, name="Ubuntu 9.04 LAMP", driver=self.driver)
        node = Node(5, None, None, None, None, self.driver, image=image)
        ret = self.driver.reboot_node(node)
        self.assertTrue(ret)

    def test_list_sizes(self):
        """
        Test ex_list_networks functionality.
        """
        sizes = self.driver.list_sizes()

        self.assertEqual(len(sizes), 3)
        size = sizes[0]
        self.assertEqual(size.id, "1")
        self.assertEqual(size.name, "small")
        self.assertEqual(size.ram, 1024)
        self.assertTrue(size.cpu is None or isinstance(size.cpu, float))
        self.assertTrue(size.vcpu is None or isinstance(size.vcpu, int))
        self.assertEqual(size.cpu, 1)
        self.assertIsNone(size.vcpu)
        self.assertIsNone(size.disk)
        self.assertIsNone(size.bandwidth)
        self.assertIsNone(size.price)
        size = sizes[1]
        self.assertEqual(size.id, "2")
        self.assertEqual(size.name, "medium")
        self.assertEqual(size.ram, 4096)
        self.assertTrue(size.cpu is None or isinstance(size.cpu, float))
        self.assertTrue(size.vcpu is None or isinstance(size.vcpu, int))
        self.assertEqual(size.cpu, 4)
        self.assertIsNone(size.vcpu)
        self.assertIsNone(size.disk)
        self.assertIsNone(size.bandwidth)
        self.assertIsNone(size.price)
        size = sizes[2]
        self.assertEqual(size.id, "3")
        self.assertEqual(size.name, "large")
        self.assertEqual(size.ram, 8192)
        self.assertTrue(size.cpu is None or isinstance(size.cpu, float))
        self.assertTrue(size.vcpu is None or isinstance(size.vcpu, int))
        self.assertEqual(size.cpu, 8)
        self.assertIsNone(size.vcpu)
        self.assertIsNone(size.disk)
        self.assertIsNone(size.bandwidth)
        self.assertIsNone(size.price)


class OpenNebula_3_6_Tests(unittest.TestCase):

    """
    OpenNebula.org test suite for OpenNebula v3.6.
    """

    def setUp(self):
        """
        Setup test environment.
        """
        OpenNebulaNodeDriver.connectionCls.conn_class = OpenNebula_3_6_MockHttp
        self.driver = OpenNebulaNodeDriver(*OPENNEBULA_PARAMS + ("3.6",), host="dummy")

    def test_create_volume(self):
        new_volume = self.driver.create_volume(1000, "test-volume")

        self.assertEqual(new_volume.id, "5")
        self.assertEqual(new_volume.size, 1000)
        self.assertEqual(new_volume.name, "test-volume")

    def test_destroy_volume(self):
        images = self.driver.list_images()

        self.assertEqual(len(images), 2)
        image = images[0]

        ret = self.driver.destroy_volume(image)
        self.assertTrue(ret)

    def test_attach_volume(self):
        nodes = self.driver.list_nodes()
        node = nodes[0]

        images = self.driver.list_images()
        image = images[0]

        ret = self.driver.attach_volume(node, image, "sda")
        self.assertTrue(ret)

    def test_detach_volume(self):
        images = self.driver.list_images()
        image = images[1]

        ret = self.driver.detach_volume(image)
        self.assertTrue(ret)

        nodes = self.driver.list_nodes()
        # node with only a single associated image
        node = nodes[1]

        ret = self.driver.detach_volume(node.image)
        self.assertFalse(ret)

    def test_list_volumes(self):
        volumes = self.driver.list_volumes()

        self.assertEqual(len(volumes), 2)

        volume = volumes[0]
        self.assertEqual(volume.id, "5")
        self.assertEqual(volume.size, 2048)
        self.assertEqual(volume.name, "Ubuntu 9.04 LAMP")

        volume = volumes[1]
        self.assertEqual(volume.id, "15")
        self.assertEqual(volume.size, 1024)
        self.assertEqual(volume.name, "Debian Sid")


class OpenNebula_3_8_Tests(unittest.TestCase):

    """
    OpenNebula.org test suite for OpenNebula v3.8.
    """

    def setUp(self):
        """
        Setup test environment.
        """
        OpenNebulaNodeDriver.connectionCls.conn_class = OpenNebula_3_8_MockHttp
        self.driver = OpenNebulaNodeDriver(*OPENNEBULA_PARAMS + ("3.8",), host="dummy")

    def test_list_sizes(self):
        """
        Test ex_list_networks functionality.
        """
        sizes = self.driver.list_sizes()

        self.assertEqual(len(sizes), 3)
        size = sizes[0]
        self.assertEqual(size.id, "1")
        self.assertEqual(size.name, "small")
        self.assertEqual(size.ram, 1024)
        self.assertEqual(size.cpu, 1)
        self.assertIsNone(size.vcpu)
        self.assertIsNone(size.disk)
        self.assertIsNone(size.bandwidth)
        self.assertIsNone(size.price)

        size = sizes[1]
        self.assertEqual(size.id, "2")
        self.assertEqual(size.name, "medium")
        self.assertEqual(size.ram, 4096)
        self.assertEqual(size.cpu, 4)
        self.assertIsNone(size.vcpu)
        self.assertIsNone(size.disk)
        self.assertIsNone(size.bandwidth)
        self.assertIsNone(size.price)

        size = sizes[2]
        self.assertEqual(size.id, "3")
        self.assertEqual(size.name, "large")
        self.assertEqual(size.ram, 8192)
        self.assertEqual(size.cpu, 8)
        self.assertIsNone(size.vcpu)
        self.assertIsNone(size.disk)
        self.assertIsNone(size.bandwidth)
        self.assertIsNone(size.price)


class OpenNebula_1_4_MockHttp(MockHttp):

    """
    Mock HTTP server for testing v1.4 of the OpenNebula.org compute driver.
    """

    fixtures = ComputeFileFixtures("opennebula_1_4")

    def _compute(self, method, url, body, headers):
        """
        Compute pool resources.
        """
        if method == "GET":
            body = self.fixtures.load("computes.xml")
            return (httplib.OK, body, {}, httplib.responses[httplib.OK])

        if method == "POST":
            body = self.fixtures.load("compute_5.xml")
            return (httplib.CREATED, body, {}, httplib.responses[httplib.CREATED])

    def _storage(self, method, url, body, headers):
        """
        Storage pool resources.
        """
        if method == "GET":
            body = self.fixtures.load("storage.xml")
            return (httplib.OK, body, {}, httplib.responses[httplib.OK])

        if method == "POST":
            body = self.fixtures.load("disk_5.xml")
            return (httplib.CREATED, body, {}, httplib.responses[httplib.CREATED])

    def _network(self, method, url, body, headers):
        """
        Network pool resources.
        """
        if method == "GET":
            body = self.fixtures.load("networks.xml")
            return (httplib.OK, body, {}, httplib.responses[httplib.OK])

        if method == "POST":
            body = self.fixtures.load("network_5.xml")
            return (httplib.CREATED, body, {}, httplib.responses[httplib.CREATED])

    def _compute_5(self, method, url, body, headers):
        """
        Compute entry resource.
        """
        if method == "GET":
            body = self.fixtures.load("compute_5.xml")
            return (httplib.OK, body, {}, httplib.responses[httplib.OK])

        if method == "PUT":
            body = ""
            return (httplib.ACCEPTED, body, {}, httplib.responses[httplib.ACCEPTED])

        if method == "DELETE":
            body = ""
            return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _compute_15(self, method, url, body, headers):
        """
        Compute entry resource.
        """
        if method == "GET":
            body = self.fixtures.load("compute_15.xml")
            return (httplib.OK, body, {}, httplib.responses[httplib.OK])

        if method == "PUT":
            body = ""
            return (httplib.ACCEPTED, body, {}, httplib.responses[httplib.ACCEPTED])

        if method == "DELETE":
            body = ""
            return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _compute_25(self, method, url, body, headers):
        """
        Compute entry resource.
        """
        if method == "GET":
            body = self.fixtures.load("compute_25.xml")
            return (httplib.OK, body, {}, httplib.responses[httplib.OK])

        if method == "PUT":
            body = ""
            return (httplib.ACCEPTED, body, {}, httplib.responses[httplib.ACCEPTED])

        if method == "DELETE":
            body = ""
            return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _storage_5(self, method, url, body, headers):
        """
        Storage entry resource.
        """
        if method == "GET":
            body = self.fixtures.load("disk_5.xml")
            return (httplib.OK, body, {}, httplib.responses[httplib.OK])

        if method == "DELETE":
            body = ""
            return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _storage_15(self, method, url, body, headers):
        """
        Storage entry resource.
        """
        if method == "GET":
            body = self.fixtures.load("disk_15.xml")
            return (httplib.OK, body, {}, httplib.responses[httplib.OK])

        if method == "DELETE":
            body = ""
            return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _network_5(self, method, url, body, headers):
        """
        Network entry resource.
        """
        if method == "GET":
            body = self.fixtures.load("network_5.xml")
            return (httplib.OK, body, {}, httplib.responses[httplib.OK])

        if method == "DELETE":
            body = ""
            return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _network_15(self, method, url, body, headers):
        """
        Network entry resource.
        """
        if method == "GET":
            body = self.fixtures.load("network_15.xml")
            return (httplib.OK, body, {}, httplib.responses[httplib.OK])

        if method == "DELETE":
            body = ""
            return (httplib.OK, body, {}, httplib.responses[httplib.OK])


class OpenNebula_2_0_MockHttp(MockHttp):

    """
    Mock HTTP server for testing v2.0 through v3.2 of the OpenNebula.org
    compute driver.
    """

    fixtures = ComputeFileFixtures("opennebula_2_0")

    def _compute(self, method, url, body, headers):
        """
        Compute pool resources.
        """
        if method == "GET":
            body = self.fixtures.load("compute_collection.xml")
            return (httplib.OK, body, {}, httplib.responses[httplib.OK])

        if method == "POST":
            body = self.fixtures.load("compute_5.xml")
            return (httplib.CREATED, body, {}, httplib.responses[httplib.CREATED])

    def _storage(self, method, url, body, headers):
        """
        Storage pool resources.
        """
        if method == "GET":
            body = self.fixtures.load("storage_collection.xml")
            return (httplib.OK, body, {}, httplib.responses[httplib.OK])

        if method == "POST":
            body = self.fixtures.load("storage_5.xml")
            return (httplib.CREATED, body, {}, httplib.responses[httplib.CREATED])

    def _network(self, method, url, body, headers):
        """
        Network pool resources.
        """
        if method == "GET":
            body = self.fixtures.load("network_collection.xml")
            return (httplib.OK, body, {}, httplib.responses[httplib.OK])

        if method == "POST":
            body = self.fixtures.load("network_5.xml")
            return (httplib.CREATED, body, {}, httplib.responses[httplib.CREATED])

    def _compute_5(self, method, url, body, headers):
        """
        Compute entry resource.
        """
        if method == "GET":
            body = self.fixtures.load("compute_5.xml")
            return (httplib.OK, body, {}, httplib.responses[httplib.OK])

        if method == "PUT":
            body = ""
            return (httplib.ACCEPTED, body, {}, httplib.responses[httplib.ACCEPTED])

        if method == "DELETE":
            body = ""
            return (httplib.NO_CONTENT, body, {}, httplib.responses[httplib.NO_CONTENT])

    def _compute_15(self, method, url, body, headers):
        """
        Compute entry resource.
        """
        if method == "GET":
            body = self.fixtures.load("compute_15.xml")
            return (httplib.OK, body, {}, httplib.responses[httplib.OK])

        if method == "PUT":
            body = ""
            return (httplib.ACCEPTED, body, {}, httplib.responses[httplib.ACCEPTED])

        if method == "DELETE":
            body = ""
            return (httplib.NO_CONTENT, body, {}, httplib.responses[httplib.NO_CONTENT])

    def _compute_25(self, method, url, body, headers):
        """
        Compute entry resource.
        """
        if method == "GET":
            body = self.fixtures.load("compute_25.xml")
            return (httplib.OK, body, {}, httplib.responses[httplib.OK])

        if method == "PUT":
            body = ""
            return (httplib.ACCEPTED, body, {}, httplib.responses[httplib.ACCEPTED])

        if method == "DELETE":
            body = ""
            return (httplib.NO_CONTENT, body, {}, httplib.responses[httplib.NO_CONTENT])

    def _storage_5(self, method, url, body, headers):
        """
        Storage entry resource.
        """
        if method == "GET":
            body = self.fixtures.load("storage_5.xml")
            return (httplib.OK, body, {}, httplib.responses[httplib.OK])

        if method == "DELETE":
            body = ""
            return (httplib.NO_CONTENT, body, {}, httplib.responses[httplib.NO_CONTENT])

    def _storage_15(self, method, url, body, headers):
        """
        Storage entry resource.
        """
        if method == "GET":
            body = self.fixtures.load("storage_15.xml")
            return (httplib.OK, body, {}, httplib.responses[httplib.OK])

        if method == "DELETE":
            body = ""
            return (httplib.NO_CONTENT, body, {}, httplib.responses[httplib.NO_CONTENT])

    def _network_5(self, method, url, body, headers):
        """
        Network entry resource.
        """
        if method == "GET":
            body = self.fixtures.load("network_5.xml")
            return (httplib.OK, body, {}, httplib.responses[httplib.OK])

        if method == "DELETE":
            body = ""
            return (httplib.NO_CONTENT, body, {}, httplib.responses[httplib.NO_CONTENT])

    def _network_15(self, method, url, body, headers):
        """
        Network entry resource.
        """
        if method == "GET":
            body = self.fixtures.load("network_15.xml")
            return (httplib.OK, body, {}, httplib.responses[httplib.OK])

        if method == "DELETE":
            body = ""
            return (httplib.NO_CONTENT, body, {}, httplib.responses[httplib.NO_CONTENT])


class OpenNebula_3_0_MockHttp(OpenNebula_2_0_MockHttp):

    """
    Mock HTTP server for testing v3.0 of the OpenNebula.org compute driver.
    """

    fixtures_3_0 = ComputeFileFixtures("opennebula_3_0")

    def _network(self, method, url, body, headers):
        """
        Network pool resources.
        """
        if method == "GET":
            body = self.fixtures_3_0.load("network_collection.xml")
            return (httplib.OK, body, {}, httplib.responses[httplib.OK])

        if method == "POST":
            body = self.fixtures.load("network_5.xml")
            return (httplib.CREATED, body, {}, httplib.responses[httplib.CREATED])

    def _network_5(self, method, url, body, headers):
        """
        Network entry resource.
        """
        if method == "GET":
            body = self.fixtures_3_0.load("network_5.xml")
            return (httplib.OK, body, {}, httplib.responses[httplib.OK])

        if method == "DELETE":
            body = ""
            return (httplib.NO_CONTENT, body, {}, httplib.responses[httplib.NO_CONTENT])

    def _network_15(self, method, url, body, headers):
        """
        Network entry resource.
        """
        if method == "GET":
            body = self.fixtures_3_0.load("network_15.xml")
            return (httplib.OK, body, {}, httplib.responses[httplib.OK])

        if method == "DELETE":
            body = ""
            return (httplib.NO_CONTENT, body, {}, httplib.responses[httplib.NO_CONTENT])


class OpenNebula_3_2_MockHttp(OpenNebula_3_0_MockHttp):

    """
    Mock HTTP server for testing v3.2 of the OpenNebula.org compute driver.
    """

    fixtures_3_2 = ComputeFileFixtures("opennebula_3_2")

    def _compute_5(self, method, url, body, headers):
        """
        Compute entry resource.
        """
        if method == "GET":
            body = self.fixtures.load("compute_5.xml")
            return (httplib.OK, body, {}, httplib.responses[httplib.OK])

        if method == "PUT":
            body = ""
            return (httplib.ACCEPTED, body, {}, httplib.responses[httplib.ACCEPTED])

        if method == "DELETE":
            body = ""
            return (httplib.NO_CONTENT, body, {}, httplib.responses[httplib.NO_CONTENT])

    def _instance_type(self, method, url, body, headers):
        """
        Instance type pool.
        """
        if method == "GET":
            body = self.fixtures_3_2.load("instance_type_collection.xml")
            return (httplib.OK, body, {}, httplib.responses[httplib.OK])


class OpenNebula_3_6_MockHttp(OpenNebula_3_2_MockHttp):

    """
    Mock HTTP server for testing v3.6 of the OpenNebula.org compute driver.
    """

    fixtures_3_6 = ComputeFileFixtures("opennebula_3_6")

    def _storage(self, method, url, body, headers):
        if method == "GET":
            body = self.fixtures.load("storage_collection.xml")
            return (httplib.OK, body, {}, httplib.responses[httplib.OK])

        if method == "POST":
            body = self.fixtures_3_6.load("storage_5.xml")
            return (httplib.CREATED, body, {}, httplib.responses[httplib.CREATED])

    def _compute_5(self, method, url, body, headers):
        if method == "GET":
            body = self.fixtures_3_6.load("compute_5.xml")
            return (httplib.OK, body, {}, httplib.responses[httplib.OK])

        if method == "PUT":
            body = ""
            return (httplib.ACCEPTED, body, {}, httplib.responses[httplib.ACCEPTED])

        if method == "DELETE":
            body = ""
            return (httplib.NO_CONTENT, body, {}, httplib.responses[httplib.NO_CONTENT])

    def _compute_5_action(self, method, url, body, headers):
        body = self.fixtures_3_6.load("compute_5.xml")
        if method == "POST":
            return (httplib.ACCEPTED, body, {}, httplib.responses[httplib.ACCEPTED])

        if method == "GET":
            return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _compute_15(self, method, url, body, headers):
        if method == "GET":
            body = self.fixtures_3_6.load("compute_15.xml")
            return (httplib.OK, body, {}, httplib.responses[httplib.OK])

        if method == "PUT":
            body = ""
            return (httplib.ACCEPTED, body, {}, httplib.responses[httplib.ACCEPTED])

        if method == "DELETE":
            body = ""
            return (httplib.NO_CONTENT, body, {}, httplib.responses[httplib.NO_CONTENT])

    def _storage_10(self, method, url, body, headers):
        """
        Storage entry resource.
        """
        if method == "GET":
            body = self.fixtures_3_6.load("disk_10.xml")
            return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _storage_15(self, method, url, body, headers):
        """
        Storage entry resource.
        """
        if method == "GET":
            body = self.fixtures_3_6.load("disk_15.xml")
            return (httplib.OK, body, {}, httplib.responses[httplib.OK])


class OpenNebula_3_8_MockHttp(OpenNebula_3_2_MockHttp):

    """
    Mock HTTP server for testing v3.8 of the OpenNebula.org compute driver.
    """

    fixtures_3_8 = ComputeFileFixtures("opennebula_3_8")

    def _instance_type(self, method, url, body, headers):
        """
        Instance type pool.
        """
        if method == "GET":
            body = self.fixtures_3_8.load("instance_type_collection.xml")
            return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _instance_type_small(self, method, url, body, headers):
        """
        Small instance type.
        """
        if method == "GET":
            body = self.fixtures_3_8.load("instance_type_small.xml")
            return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _instance_type_medium(self, method, url, body, headers):
        """
        Medium instance type pool.
        """
        if method == "GET":
            body = self.fixtures_3_8.load("instance_type_medium.xml")
            return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _instance_type_large(self, method, url, body, headers):
        """
        Large instance type pool.
        """
        if method == "GET":
            body = self.fixtures_3_8.load("instance_type_large.xml")
            return (httplib.OK, body, {}, httplib.responses[httplib.OK])


if __name__ == "__main__":
    sys.exit(unittest.main())



# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


__all__ = ["KubernetesAuthTestCaseMixin"]

import os
import base64

from libcloud.utils.py3 import b
from libcloud.common.kubernetes import (
    KubernetesTLSAuthConnection,
    KubernetesBasicAuthConnection,
    KubernetesTokenAuthConnection,
)

KEY_FILE = os.path.abspath(
    os.path.join(os.path.dirname(__file__), "../compute/fixtures/azure/libcloud.pem")
)
CERT_FILE = os.path.abspath(
    os.path.join(os.path.dirname(__file__), "../loadbalancer/fixtures/nttcis/denis.crt")
)
CA_CERT_FILE = os.path.abspath(
    os.path.join(os.path.dirname(__file__), "../loadbalancer/fixtures/nttcis/chain.crt")
)


class KubernetesAuthTestCaseMixin:
    """
    Test class mixin which tests different type of Kubernetes authentication
    mechanisms (client cert, token, basic auth).

    It's to be used with all the drivers which inherit from KubernetesDriverMixin.
    """

    def test_http_basic_auth(self):
        driver = self.driver_cls(key="username", secret="password")
        self.assertEqual(driver.connectionCls, KubernetesBasicAuthConnection)
        self.assertEqual(driver.connection.user_id, "username")
        self.assertEqual(driver.connection.key, "password")

        auth_string = base64.b64encode(b("{}:{}".format("username", "password"))).decode("utf-8")

        headers = driver.connection.add_default_headers({})
        self.assertEqual(headers["Content-Type"], "application/json")
        self.assertEqual(headers["Authorization"], "Basic %s" % (auth_string))

    def test_cert_auth(self):
        # key_file provided, but not cert_file
        expected_msg = "Both key and certificate files are needed"
        self.assertRaisesRegex(
            ValueError,
            expected_msg,
            self.driver_cls,
            key_file=KEY_FILE,
            ca_cert=CA_CERT_FILE,
        )

        # cert_file provided, but not key_file
        expected_msg = "Both key and certificate files are needed"
        self.assertRaisesRegex(
            ValueError,
            expected_msg,
            self.driver_cls,
            cert_file=CERT_FILE,
            ca_cert=CA_CERT_FILE,
        )

        # ca_cert argument specified
        driver = self.driver_cls(key_file=KEY_FILE, cert_file=CERT_FILE, ca_cert=CA_CERT_FILE)
        self.assertEqual(driver.connectionCls, KubernetesTLSAuthConnection)
        self.assertEqual(driver.connection.key_file, KEY_FILE)
        self.assertEqual(driver.connection.cert_file, CERT_FILE)
        self.assertEqual(driver.connection.connection.ca_cert, CA_CERT_FILE)

        headers = driver.connection.add_default_headers({})
        self.assertEqual(headers["Content-Type"], "application/json")

        # ca_cert argument not specified
        driver = self.driver_cls(key_file=KEY_FILE, cert_file=CERT_FILE, ca_cert=None)
        self.assertEqual(driver.connectionCls, KubernetesTLSAuthConnection)
        self.assertEqual(driver.connection.key_file, KEY_FILE)
        self.assertEqual(driver.connection.cert_file, CERT_FILE)
        self.assertEqual(driver.connection.connection.ca_cert, False)

        headers = driver.connection.add_default_headers({})
        self.assertEqual(headers["Content-Type"], "application/json")

    def test_bearer_token_auth(self):
        driver = self.driver_cls(ex_token_bearer_auth=True, key="foobar")
        self.assertEqual(driver.connectionCls, KubernetesTokenAuthConnection)
        self.assertEqual(driver.connection.key, "foobar")

        headers = driver.connection.add_default_headers({})
        self.assertEqual(headers["Content-Type"], "application/json")
        self.assertEqual(headers["Authorization"], "Bearer %s" % ("foobar"))

    def test_host_sanitization(self):
        driver = self.driver_cls(host="example.com")
        self.assertEqual(driver.connection.host, "example.com")

        driver = self.driver_cls(host="http://example.com")
        self.assertEqual(driver.connection.host, "example.com")

        driver = self.driver_cls(host="https://example.com")
        self.assertEqual(driver.connection.host, "example.com")



# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys
import unittest

from libcloud.test import LibcloudTestCase
from libcloud.common import aliyun
from libcloud.common.aliyun import AliyunRequestSignerAlgorithmV1_0


class AliyunRequestSignerAlgorithmV1_0TestCase(LibcloudTestCase):
    def setUp(self):
        self.signer = AliyunRequestSignerAlgorithmV1_0("testid", "testsecret", "1.0")

    def test_sign_request(self):
        params = {
            "TimeStamp": "2012-12-26T10:33:56Z",
            "Format": "XML",
            "AccessKeyId": "testid",
            "Action": "DescribeRegions",
            "SignatureMethod": "HMAC-SHA1",
            "RegionId": "region1",
            "SignatureNonce": "NwDAxvLU6tFE0DVb",
            "Version": "2014-05-26",
            "SignatureVersion": "1.0",
        }
        method = "GET"
        path = "/"

        expected = "K9fCVP6Jrklpd3rLYKh1pfrrFNo="
        self.assertEqual(expected, self.signer._sign_request(params, method, path))


class AliyunCommonTestCase(LibcloudTestCase):
    def test_percent_encode(self):
        data = {"abc": "abc", " *~": "%20%2A~"}
        for key in data:
            self.assertEqual(data[key], aliyun._percent_encode(key))


if __name__ == "__main__":
    sys.exit(unittest.main())



# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import sys

from libcloud.test import MockHttp, unittest
from libcloud.utils.py3 import httplib
from libcloud.common.types import InvalidCredsError
from libcloud.test.secrets import DIMENSIONDATA_PARAMS
from libcloud.loadbalancer.base import Member, Algorithm, LoadBalancer
from libcloud.loadbalancer.types import State
from libcloud.test.file_fixtures import LoadBalancerFileFixtures
from libcloud.common.dimensiondata import (
    DimensionDataPool,
    DimensionDataVIPNode,
    DimensionDataPoolMember,
)
from libcloud.loadbalancer.drivers.dimensiondata import DimensionDataLBDriver as DimensionData


class DimensionData_v2_4_Tests(unittest.TestCase):
    def setUp(self):
        DimensionData.connectionCls.active_api_version = "2.4"
        DimensionData.connectionCls.conn_class = DimensionDataMockHttp
        DimensionDataMockHttp.type = None
        self.driver = DimensionData(*DIMENSIONDATA_PARAMS)

    def test_invalid_region(self):
        with self.assertRaises(ValueError):
            self.driver = DimensionData(*DIMENSIONDATA_PARAMS, region="blah")

    def test_invalid_creds(self):
        DimensionDataMockHttp.type = "UNAUTHORIZED"
        with self.assertRaises(InvalidCredsError):
            self.driver.list_balancers()

    def test_create_balancer(self):
        self.driver.ex_set_current_network_domain("1234")
        members = []
        members.append(Member(id=None, ip="1.2.3.4", port=80))

        balancer = self.driver.create_balancer(
            name="test",
            port=80,
            protocol="http",
            algorithm=Algorithm.ROUND_ROBIN,
            members=members,
            ex_listener_ip_address="5.6.7.8",
        )
        self.assertEqual(balancer.name, "test")
        self.assertEqual(balancer.id, "8334f461-0df0-42d5-97eb-f4678eb26bea")
        self.assertEqual(balancer.ip, "165.180.12.22")
        self.assertEqual(balancer.port, 80)
        self.assertEqual(balancer.extra["pool_id"], "9e6b496d-5261-4542-91aa-b50c7f569c54")
        self.assertEqual(balancer.extra["network_domain_id"], "1234")
        self.assertEqual(balancer.extra["listener_ip_address"], "5.6.7.8")

    def test_create_balancer_with_defaults(self):
        self.driver.ex_set_current_network_domain("1234")

        balancer = self.driver.create_balancer(
            name="test", port=None, protocol=None, algorithm=None, members=None
        )
        self.assertEqual(balancer.name, "test")
        self.assertEqual(balancer.id, "8334f461-0df0-42d5-97eb-f4678eb26bea")
        self.assertEqual(balancer.ip, "165.180.12.22")
        self.assertIsNone(balancer.port)
        self.assertEqual(balancer.extra["pool_id"], "9e6b496d-5261-4542-91aa-b50c7f569c54")
        self.assertEqual(balancer.extra["network_domain_id"], "1234")

    def test_create_balancer_no_members(self):
        self.driver.ex_set_current_network_domain("1234")
        members = None

        balancer = self.driver.create_balancer(
            name="test",
            port=80,
            protocol="http",
            algorithm=Algorithm.ROUND_ROBIN,
            members=members,
        )
        self.assertEqual(balancer.name, "test")
        self.assertEqual(balancer.id, "8334f461-0df0-42d5-97eb-f4678eb26bea")
        self.assertEqual(balancer.ip, "165.180.12.22")
        self.assertEqual(balancer.port, 80)
        self.assertEqual(balancer.extra["pool_id"], "9e6b496d-5261-4542-91aa-b50c7f569c54")
        self.assertEqual(balancer.extra["network_domain_id"], "1234")

    def test_create_balancer_empty_members(self):
        self.driver.ex_set_current_network_domain("1234")
        members = []

        balancer = self.driver.create_balancer(
            name="test",
            port=80,
            protocol="http",
            algorithm=Algorithm.ROUND_ROBIN,
            members=members,
        )
        self.assertEqual(balancer.name, "test")
        self.assertEqual(balancer.id, "8334f461-0df0-42d5-97eb-f4678eb26bea")
        self.assertEqual(balancer.ip, "165.180.12.22")
        self.assertEqual(balancer.port, 80)
        self.assertEqual(balancer.extra["pool_id"], "9e6b496d-5261-4542-91aa-b50c7f569c54")
        self.assertEqual(balancer.extra["network_domain_id"], "1234")

    def test_list_balancers(self):
        bal = self.driver.list_balancers()
        self.assertEqual(bal[0].name, "myProduction.Virtual.Listener")
        self.assertEqual(bal[0].id, "6115469d-a8bb-445b-bb23-d23b5283f2b9")
        self.assertEqual(bal[0].port, "8899")
        self.assertEqual(bal[0].ip, "165.180.12.22")
        self.assertEqual(bal[0].state, State.RUNNING)

    def test_balancer_list_members(self):
        extra = {
            "pool_id": "4d360b1f-bc2c-4ab7-9884-1f03ba2768f7",
            "network_domain_id": "1234",
        }
        balancer = LoadBalancer(
            id="234",
            name="test",
            state=State.RUNNING,
            ip="1.2.3.4",
            port=1234,
            driver=self.driver,
            extra=extra,
        )
        members = self.driver.balancer_list_members(balancer)
        self.assertEqual(2, len(members))
        self.assertEqual(members[0].ip, "10.0.3.13")
        self.assertEqual(members[0].id, "3dd806a2-c2c8-4c0c-9a4f-5219ea9266c0")
        self.assertEqual(members[0].port, 9889)

    def test_balancer_attach_member(self):
        extra = {
            "pool_id": "4d360b1f-bc2c-4ab7-9884-1f03ba2768f7",
            "network_domain_id": "1234",
        }
        balancer = LoadBalancer(
            id="234",
            name="test",
            state=State.RUNNING,
            ip="1.2.3.4",
            port=1234,
            driver=self.driver,
            extra=extra,
        )
        member = Member(id=None, ip="112.12.2.2", port=80, balancer=balancer, extra=None)
        member = self.driver.balancer_attach_member(balancer, member)
        self.assertEqual(member.id, "3dd806a2-c2c8-4c0c-9a4f-5219ea9266c0")

    def test_balancer_attach_member_without_port(self):
        extra = {
            "pool_id": "4d360b1f-bc2c-4ab7-9884-1f03ba2768f7",
            "network_domain_id": "1234",
        }
        balancer = LoadBalancer(
            id="234",
            name="test",
            state=State.RUNNING,
            ip="1.2.3.4",
            port=1234,
            driver=self.driver,
            extra=extra,
        )
        member = Member(id=None, ip="112.12.2.2", port=None, balancer=balancer, extra=None)
        member = self.driver.balancer_attach_member(balancer, member)
        self.assertEqual(member.id, "3dd806a2-c2c8-4c0c-9a4f-5219ea9266c0")
        self.assertIsNone(member.port)

    def test_balancer_detach_member(self):
        extra = {
            "pool_id": "4d360b1f-bc2c-4ab7-9884-1f03ba2768f7",
            "network_domain_id": "1234",
        }
        balancer = LoadBalancer(
            id="234",
            name="test",
            state=State.RUNNING,
            ip="1.2.3.4",
            port=1234,
            driver=self.driver,
            extra=extra,
        )
        member = Member(
            id="3dd806a2-c2c8-4c0c-9a4f-5219ea9266c0",
            ip="112.12.2.2",
            port=80,
            balancer=balancer,
            extra=None,
        )
        result = self.driver.balancer_detach_member(balancer, member)
        self.assertEqual(result, True)

    def test_destroy_balancer(self):
        extra = {
            "pool_id": "4d360b1f-bc2c-4ab7-9884-1f03ba2768f7",
            "network_domain_id": "1234",
        }
        balancer = LoadBalancer(
            id="234",
            name="test",
            state=State.RUNNING,
            ip="1.2.3.4",
            port=1234,
            driver=self.driver,
            extra=extra,
        )
        response = self.driver.destroy_balancer(balancer)
        self.assertEqual(response, True)

    def test_set_get_network_domain_id(self):
        self.driver.ex_set_current_network_domain("1234")
        nwd = self.driver.ex_get_current_network_domain()
        self.assertEqual(nwd, "1234")

    def test_ex_create_pool_member(self):
        pool = DimensionDataPool(
            id="4d360b1f-bc2c-4ab7-9884-1f03ba2768f7",
            name="test",
            description="test",
            status=State.RUNNING,
            health_monitor_id=None,
            load_balance_method=None,
            service_down_action=None,
            slow_ramp_time=None,
        )
        node = DimensionDataVIPNode(id="2344", name="test", status=State.RUNNING, ip="123.23.3.2")
        member = self.driver.ex_create_pool_member(pool=pool, node=node, port=80)
        self.assertEqual(member.id, "3dd806a2-c2c8-4c0c-9a4f-5219ea9266c0")
        self.assertEqual(member.name, "10.0.3.13")
        self.assertEqual(member.ip, "123.23.3.2")

    def test_ex_create_node(self):
        node = self.driver.ex_create_node(
            network_domain_id="12345",
            name="test",
            ip="123.12.32.2",
            ex_description="",
            connection_limit=25000,
            connection_rate_limit=2000,
        )
        self.assertEqual(node.name, "myProductionNode.1")
        self.assertEqual(node.id, "9e6b496d-5261-4542-91aa-b50c7f569c54")

    def test_ex_create_pool(
        self,
    ):
        pool = self.driver.ex_create_pool(
            network_domain_id="1234",
            name="test",
            balancer_method="ROUND_ROBIN",
            ex_description="test",
            service_down_action="NONE",
            slow_ramp_time=30,
        )
        self.assertEqual(pool.id, "9e6b496d-5261-4542-91aa-b50c7f569c54")
        self.assertEqual(pool.name, "test")
        self.assertEqual(pool.status, State.RUNNING)

    def test_ex_create_virtual_listener(self):
        listener = self.driver.ex_create_virtual_listener(
            network_domain_id="12345",
            name="test",
            ex_description="test",
            port=80,
            pool=DimensionDataPool(
                id="1234",
                name="test",
                description="test",
                status=State.RUNNING,
                health_monitor_id=None,
                load_balance_method=None,
                service_down_action=None,
                slow_ramp_time=None,
            ),
        )
        self.assertEqual(listener.id, "8334f461-0df0-42d5-97eb-f4678eb26bea")
        self.assertEqual(listener.name, "test")

    def test_ex_create_virtual_listener_unusual_port(self):
        listener = self.driver.ex_create_virtual_listener(
            network_domain_id="12345",
            name="test",
            ex_description="test",
            port=8900,
            pool=DimensionDataPool(
                id="1234",
                name="test",
                description="test",
                status=State.RUNNING,
                health_monitor_id=None,
                load_balance_method=None,
                service_down_action=None,
                slow_ramp_time=None,
            ),
        )
        self.assertEqual(listener.id, "8334f461-0df0-42d5-97eb-f4678eb26bea")
        self.assertEqual(listener.name, "test")

    def test_ex_create_virtual_listener_without_port(self):
        listener = self.driver.ex_create_virtual_listener(
            network_domain_id="12345",
            name="test",
            ex_description="test",
            pool=DimensionDataPool(
                id="1234",
                name="test",
                description="test",
                status=State.RUNNING,
                health_monitor_id=None,
                load_balance_method=None,
                service_down_action=None,
                slow_ramp_time=None,
            ),
        )
        self.assertEqual(listener.id, "8334f461-0df0-42d5-97eb-f4678eb26bea")
        self.assertEqual(listener.name, "test")

    def test_ex_create_virtual_listener_without_pool(self):
        listener = self.driver.ex_create_virtual_listener(
            network_domain_id="12345", name="test", ex_description="test"
        )
        self.assertEqual(listener.id, "8334f461-0df0-42d5-97eb-f4678eb26bea")
        self.assertEqual(listener.name, "test")

    def test_get_balancer(self):
        bal = self.driver.get_balancer("6115469d-a8bb-445b-bb23-d23b5283f2b9")
        self.assertEqual(bal.name, "myProduction.Virtual.Listener")
        self.assertEqual(bal.id, "6115469d-a8bb-445b-bb23-d23b5283f2b9")
        self.assertEqual(bal.port, "8899")
        self.assertEqual(bal.ip, "165.180.12.22")
        self.assertEqual(bal.state, State.RUNNING)

    def test_list_protocols(self):
        protocols = self.driver.list_protocols()
        self.assertNotEqual(0, len(protocols))

    def test_ex_get_nodes(self):
        nodes = self.driver.ex_get_nodes()
        self.assertEqual(2, len(nodes))
        self.assertEqual(nodes[0].name, "ProductionNode.1")
        self.assertEqual(nodes[0].id, "34de6ed6-46a4-4dae-a753-2f8d3840c6f9")
        self.assertEqual(nodes[0].ip, "10.10.10.101")

    def test_ex_get_node(self):
        node = self.driver.ex_get_node("34de6ed6-46a4-4dae-a753-2f8d3840c6f9")
        self.assertEqual(node.name, "ProductionNode.2")
        self.assertEqual(node.id, "34de6ed6-46a4-4dae-a753-2f8d3840c6f9")
        self.assertEqual(node.ip, "10.10.10.101")

    def test_ex_update_node(self):
        node = self.driver.ex_get_node("34de6ed6-46a4-4dae-a753-2f8d3840c6f9")
        node.connection_limit = "100"
        result = self.driver.ex_update_node(node)
        self.assertEqual(result.connection_limit, "100")

    def test_ex_destroy_node(self):
        result = self.driver.ex_destroy_node("34de6ed6-46a4-4dae-a753-2f8d3840c6f9")
        self.assertTrue(result)

    def test_ex_set_node_state(self):
        node = self.driver.ex_get_node("34de6ed6-46a4-4dae-a753-2f8d3840c6f9")
        result = self.driver.ex_set_node_state(node, False)
        self.assertEqual(result.connection_limit, "10000")

    def test_ex_get_pools(self):
        pools = self.driver.ex_get_pools()
        self.assertNotEqual(0, len(pools))
        self.assertEqual(pools[0].name, "myDevelopmentPool.1")
        self.assertEqual(pools[0].id, "4d360b1f-bc2c-4ab7-9884-1f03ba2768f7")

    def test_ex_get_pool(self):
        pool = self.driver.ex_get_pool("4d360b1f-bc2c-4ab7-9884-1f03ba2768f7")
        self.assertEqual(pool.name, "myDevelopmentPool.1")
        self.assertEqual(pool.id, "4d360b1f-bc2c-4ab7-9884-1f03ba2768f7")

    def test_ex_update_pool(self):
        pool = self.driver.ex_get_pool("4d360b1f-bc2c-4ab7-9884-1f03ba2768f7")
        pool.slow_ramp_time = "120"
        result = self.driver.ex_update_pool(pool)
        self.assertTrue(result)

    def test_ex_destroy_pool(self):
        response = self.driver.ex_destroy_pool(
            pool=DimensionDataPool(
                id="4d360b1f-bc2c-4ab7-9884-1f03ba2768f7",
                name="test",
                description="test",
                status=State.RUNNING,
                health_monitor_id=None,
                load_balance_method=None,
                service_down_action=None,
                slow_ramp_time=None,
            )
        )
        self.assertTrue(response)

    def test_get_pool_members(self):
        members = self.driver.ex_get_pool_members("4d360b1f-bc2c-4ab7-9884-1f03ba2768f7")
        self.assertEqual(2, len(members))
        self.assertEqual(members[0].id, "3dd806a2-c2c8-4c0c-9a4f-5219ea9266c0")
        self.assertEqual(members[0].name, "10.0.3.13")
        self.assertEqual(members[0].status, "NORMAL")
        self.assertEqual(members[0].ip, "10.0.3.13")
        self.assertEqual(members[0].port, 9889)
        self.assertEqual(members[0].node_id, "3c207269-e75e-11e4-811f-005056806999")

    def test_get_pool_member(self):
        member = self.driver.ex_get_pool_member("3dd806a2-c2c8-4c0c-9a4f-5219ea9266c0")
        self.assertEqual(member.id, "3dd806a2-c2c8-4c0c-9a4f-5219ea9266c0")
        self.assertEqual(member.name, "10.0.3.13")
        self.assertEqual(member.status, "NORMAL")
        self.assertEqual(member.ip, "10.0.3.13")
        self.assertEqual(member.port, 9889)

    def test_set_pool_member_state(self):
        member = self.driver.ex_get_pool_member("3dd806a2-c2c8-4c0c-9a4f-5219ea9266c0")
        result = self.driver.ex_set_pool_member_state(member, True)
        self.assertTrue(result)

    def test_ex_destroy_pool_member(self):
        response = self.driver.ex_destroy_pool_member(
            member=DimensionDataPoolMember(
                id="",
                name="test",
                status=State.RUNNING,
                ip="1.2.3.4",
                port=80,
                node_id="3c207269-e75e-11e4-811f-005056806999",
            ),
            destroy_node=False,
        )
        self.assertTrue(response)

    def test_ex_destroy_pool_member_with_node(self):
        response = self.driver.ex_destroy_pool_member(
            member=DimensionDataPoolMember(
                id="",
                name="test",
                status=State.RUNNING,
                ip="1.2.3.4",
                port=80,
                node_id="34de6ed6-46a4-4dae-a753-2f8d3840c6f9",
            ),
            destroy_node=True,
        )
        self.assertTrue(response)

    def test_ex_get_default_health_monitors(self):
        monitors = self.driver.ex_get_default_health_monitors(
            "4d360b1f-bc2c-4ab7-9884-1f03ba2768f7"
        )
        self.assertEqual(len(monitors), 6)
        self.assertEqual(monitors[0].id, "01683574-d487-11e4-811f-005056806999")
        self.assertEqual(monitors[0].name, "CCDEFAULT.Http")
        self.assertFalse(monitors[0].node_compatible)
        self.assertTrue(monitors[0].pool_compatible)

    def test_ex_get_default_persistence_profiles(self):
        profiles = self.driver.ex_get_default_persistence_profiles(
            "4d360b1f-bc2c-4ab7-9884-1f03ba2768f7"
        )
        self.assertEqual(len(profiles), 4)
        self.assertEqual(profiles[0].id, "a34ca024-f3db-11e4-b010-005056806999")
        self.assertEqual(profiles[0].name, "CCDEFAULT.Cookie")
        self.assertEqual(profiles[0].fallback_compatible, False)
        self.assertEqual(len(profiles[0].compatible_listeners), 1)
        self.assertEqual(profiles[0].compatible_listeners[0].type, "PERFORMANCE_LAYER_4")

    def test_ex_get_default_irules(self):
        irules = self.driver.ex_get_default_irules("4d360b1f-bc2c-4ab7-9884-1f03ba2768f7")
        self.assertEqual(len(irules), 4)
        self.assertEqual(irules[0].id, "2b20cb2c-ffdc-11e4-b010-005056806999")
        self.assertEqual(irules[0].name, "CCDEFAULT.HttpsRedirect")
        self.assertEqual(len(irules[0].compatible_listeners), 1)
        self.assertEqual(irules[0].compatible_listeners[0].type, "PERFORMANCE_LAYER_4")


class DimensionDataMockHttp(MockHttp):
    fixtures = LoadBalancerFileFixtures("dimensiondata")

    def _oec_0_9_myaccount_UNAUTHORIZED(self, method, url, body, headers):
        return (httplib.UNAUTHORIZED, "", {}, httplib.responses[httplib.UNAUTHORIZED])

    def _oec_0_9_myaccount(self, method, url, body, headers):
        body = self.fixtures.load("oec_0_9_myaccount.xml")
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _oec_0_9_myaccount_INPROGRESS(self, method, url, body, headers):
        body = self.fixtures.load("oec_0_9_myaccount.xml")
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _caas_2_4_8a8f6abc_2745_4d8a_9cbc_8dabe5a7d0e4_networkDomainVip_virtualListener(
        self, method, url, body, headers
    ):
        body = self.fixtures.load("networkDomainVip_virtualListener.xml")
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _caas_2_4_8a8f6abc_2745_4d8a_9cbc_8dabe5a7d0e4_networkDomainVip_virtualListener_6115469d_a8bb_445b_bb23_d23b5283f2b9(
        self, method, url, body, headers
    ):
        body = self.fixtures.load(
            "networkDomainVip_virtualListener_6115469d_a8bb_445b_bb23_d23b5283f2b9.xml"
        )
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _caas_2_4_8a8f6abc_2745_4d8a_9cbc_8dabe5a7d0e4_networkDomainVip_pool(
        self, method, url, body, headers
    ):
        body = self.fixtures.load("networkDomainVip_pool.xml")
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _caas_2_4_8a8f6abc_2745_4d8a_9cbc_8dabe5a7d0e4_networkDomainVip_pool_4d360b1f_bc2c_4ab7_9884_1f03ba2768f7(
        self, method, url, body, headers
    ):
        body = self.fixtures.load("networkDomainVip_pool_4d360b1f_bc2c_4ab7_9884_1f03ba2768f7.xml")
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _caas_2_4_8a8f6abc_2745_4d8a_9cbc_8dabe5a7d0e4_networkDomainVip_poolMember(
        self, method, url, body, headers
    ):
        body = self.fixtures.load("networkDomainVip_poolMember.xml")
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _caas_2_4_8a8f6abc_2745_4d8a_9cbc_8dabe5a7d0e4_networkDomainVip_poolMember_3dd806a2_c2c8_4c0c_9a4f_5219ea9266c0(
        self, method, url, body, headers
    ):
        body = self.fixtures.load(
            "networkDomainVip_poolMember_3dd806a2_c2c8_4c0c_9a4f_5219ea9266c0.xml"
        )
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _caas_2_4_8a8f6abc_2745_4d8a_9cbc_8dabe5a7d0e4_networkDomainVip_createPool(
        self, method, url, body, headers
    ):
        body = self.fixtures.load("networkDomainVip_createPool.xml")
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _caas_2_4_8a8f6abc_2745_4d8a_9cbc_8dabe5a7d0e4_networkDomainVip_createNode(
        self, method, url, body, headers
    ):
        body = self.fixtures.load("networkDomainVip_createNode.xml")
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _caas_2_4_8a8f6abc_2745_4d8a_9cbc_8dabe5a7d0e4_networkDomainVip_addPoolMember(
        self, method, url, body, headers
    ):
        body = self.fixtures.load("networkDomainVip_addPoolMember.xml")
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _caas_2_4_8a8f6abc_2745_4d8a_9cbc_8dabe5a7d0e4_networkDomainVip_createVirtualListener(
        self, method, url, body, headers
    ):
        body = self.fixtures.load("networkDomainVip_createVirtualListener.xml")
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _caas_2_4_8a8f6abc_2745_4d8a_9cbc_8dabe5a7d0e4_networkDomainVip_removePoolMember(
        self, method, url, body, headers
    ):
        body = self.fixtures.load("networkDomainVip_removePoolMember.xml")
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _caas_2_4_8a8f6abc_2745_4d8a_9cbc_8dabe5a7d0e4_networkDomainVip_deleteVirtualListener(
        self, method, url, body, headers
    ):
        body = self.fixtures.load("networkDomainVip_deleteVirtualListener.xml")
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _caas_2_4_8a8f6abc_2745_4d8a_9cbc_8dabe5a7d0e4_networkDomainVip_deletePool(
        self, method, url, body, headers
    ):
        body = self.fixtures.load("networkDomainVip_deletePool.xml")
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _caas_2_4_8a8f6abc_2745_4d8a_9cbc_8dabe5a7d0e4_networkDomainVip_deleteNode(
        self, method, url, body, headers
    ):
        body = self.fixtures.load("networkDomainVip_deleteNode.xml")
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _caas_2_4_8a8f6abc_2745_4d8a_9cbc_8dabe5a7d0e4_networkDomainVip_node(
        self, method, url, body, headers
    ):
        body = self.fixtures.load("networkDomainVip_node.xml")
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _caas_2_4_8a8f6abc_2745_4d8a_9cbc_8dabe5a7d0e4_networkDomainVip_node_34de6ed6_46a4_4dae_a753_2f8d3840c6f9(
        self, method, url, body, headers
    ):
        body = self.fixtures.load("networkDomainVip_node_34de6ed6_46a4_4dae_a753_2f8d3840c6f9.xml")
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _caas_2_4_8a8f6abc_2745_4d8a_9cbc_8dabe5a7d0e4_networkDomainVip_editNode(
        self, method, url, body, headers
    ):
        body = self.fixtures.load("networkDomainVip_editNode.xml")
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _caas_2_4_8a8f6abc_2745_4d8a_9cbc_8dabe5a7d0e4_networkDomainVip_editPool(
        self, method, url, body, headers
    ):
        body = self.fixtures.load("networkDomainVip_editPool.xml")
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _caas_2_4_8a8f6abc_2745_4d8a_9cbc_8dabe5a7d0e4_networkDomainVip_editPoolMember(
        self, method, url, body, headers
    ):
        body = self.fixtures.load("networkDomainVip_editPoolMember.xml")
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _caas_2_4_8a8f6abc_2745_4d8a_9cbc_8dabe5a7d0e4_networkDomainVip_defaultHealthMonitor(
        self, method, url, body, headers
    ):
        body = self.fixtures.load("networkDomainVip_defaultHealthMonitor.xml")
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _caas_2_4_8a8f6abc_2745_4d8a_9cbc_8dabe5a7d0e4_networkDomainVip_defaultPersistenceProfile(
        self, method, url, body, headers
    ):
        body = self.fixtures.load("networkDomainVip_defaultPersistenceProfile.xml")
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])

    def _caas_2_4_8a8f6abc_2745_4d8a_9cbc_8dabe5a7d0e4_networkDomainVip_defaultIrule(
        self, method, url, body, headers
    ):
        body = self.fixtures.load("networkDomainVip_defaultIrule.xml")
        return (httplib.OK, body, {}, httplib.responses[httplib.OK])


if __name__ == "__main__":
    sys.exit(unittest.main())



# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pytest

from libcloud.storage.drivers.dummy import DummyStorageDriver


@pytest.fixture
def driver():
    return DummyStorageDriver("key", "id")


@pytest.fixture
def container_with_contents(driver):
    container_name = "test"
    object_name = "test.dat"
    container = driver.create_container(container_name=container_name)
    driver.upload_object(__file__, container=container, object_name=object_name)
    return container_name, object_name


def test_list_container_objects(driver, container_with_contents):
    container_name, object_name = container_with_contents
    container = driver.get_container(container_name)

    objects = driver.list_container_objects(container=container)

    assert any(o for o in objects if o.name == object_name)


def test_list_container_objects_filter_by_prefix(driver, container_with_contents):
    container_name, object_name = container_with_contents
    container = driver.get_container(container_name)

    objects = driver.list_container_objects(container=container, prefix=object_name[:3])
    assert any(o for o in objects if o.name == object_name)

    objects = driver.list_container_objects(container=container, prefix="does-not-exist.dat")
    assert not objects
